@@ -1,10 +1,15 @@
 #include "volumecontrol.h"
+#include "config.h"
 
 #ifdef USING_OSS
-#include <sys/soundcard.h>
+    #include <sys/ioctl.h>
+    #ifdef HAVE_SYS_SOUNDCARD_H
+        #include <sys/soundcard.h>
+    #elif HAVE_SOUNDCARD_H
+        #include <soundcard.h>
+    #endif
 #endif
 
-#include <sys/ioctl.h>
 #include <fcntl.h>
 #include <cstdio>
 #include <unistd.h>
@@ -16,6 +21,8 @@ using namespace std;
 
 VolumeControl::VolumeControl(bool setstartingvolume)
 {
+    (void) setstartingvolume;
+
     mixerfd = -1;
     volume = 0;
 
@@ -96,6 +103,8 @@ int VolumeControl::GetCurrentVolume(void)
 
 void VolumeControl::SetCurrentVolume(int value)
 {
+    (void) value;
+
 #ifdef USING_OSS
     volume = value;
 
@@ -133,6 +142,8 @@ void VolumeControl::AdjustCurrentVolume(int change)
 
 void VolumeControl::SetMute(bool on)
 {
+    (void) on;
+
 #ifdef USING_OSS
     int realvol;
 
@@ -194,8 +205,7 @@ kMuteState VolumeControl::IterateMutedChannels(void)
         if (ret < 0)
             perror("IterateMutedChannels:");
     }
+#endif // USING_OSS
 
-    return (current_mute_state);
-
-#endif
+    return current_mute_state;
 }
