@@ -24,7 +24,7 @@ extern "C" {
 
 class NuppelVideoPlayer;
 
-class DVDRingBufferPriv
+class MPUBLIC DVDRingBufferPriv
 {
   public:
     DVDRingBufferPriv();
@@ -52,18 +52,19 @@ class DVDRingBufferPriv
     void ReleaseMenuButton(void);
 
     bool IgnoringStillorWait(void) { return skipstillorwait; }
-    long long GetCellStartPos(void);
-    uint ButtonPosX(void) { return hl_startx; }
-    uint ButtonPosY(void) { return hl_starty; }
+    uint ButtonPosX(void)   { return hl_startx; }
+    uint ButtonPosY(void)   { return hl_starty; }
+    uint ButtonWidth(void)  { return hl_width; }
+    uint ButtonHeight(void) { return hl_height; }
     uint GetAudioLanguage(int id);
-    uint GetSubtitleLanguage(int id);
+    int  GetSubTrackNum(uint key);
+    int  GetAudioTrackNum(uint key);
+    uint GetSubtitleLanguage(int key);
     void SetMenuPktPts(long long pts) { menupktpts = pts; }
     long long GetMenuPktPts(void) { return menupktpts; }
     bool DecodeSubtitles(AVSubtitle * sub, int * gotSubtitles, 
                          const uint8_t * buf, int buf_size);
-    void GetNameAndSerialNum(const char **_name, const char **_serial)
-        { (*_name) = dvdname; (*_serial) = serialnumber; }
-
+    bool GetNameAndSerialNum(QString& _name, QString& _serialnum);
     bool JumpToTitle(void) { return jumptotitle; }
     double GetFrameRate(void);
     bool StartOfTitle(void) { return (part == 0); }
@@ -73,19 +74,19 @@ class DVDRingBufferPriv
     int GetCellID(void) { return cellid; }
     int GetVobID(void)  { return vobid; }
     bool IsSameChapter(int tmpcellid, int tmpvobid);
-    
+
     // commands
     bool OpenFile(const QString &filename);
     void PlayTitleAndPart(int _title, int _part) 
         { dvdnav_part_play(dvdnav, _title, _part); }
-    void close(void);
+    void CloseDVD(void);
     bool nextTrack(void);
     void prevTrack(void);
     int  safe_read(void *data, unsigned sz);
-    long long Seek(long long pos, int whence);
+    long long NormalSeek(long long time);
     void SkipStillFrame(void);
     void WaitSkip(void);
-    void GoToMenu(const QString str);
+    bool GoToMenu(const QString str);
     void GoToNextProgram(void);
     void GoToPreviousProgram(void);
     void MoveButtonLeft(void);
@@ -95,24 +96,32 @@ class DVDRingBufferPriv
     void ActivateButton(void);
     int NumMenuButtons(void) const;
     void IgnoreStillOrWait(bool skip) { skipstillorwait = skip; }
-    uint GetCurrentTime(void);
+    void InStillFrame(bool change) { cellHasStillFrame = change; }
+    uint GetCurrentTime(void) { return (currentTime / 90000); }
+    uint TitleTimeLeft(void);
     void  SetTrack(uint type, int trackNo);
     int   GetTrack(uint type);
     uint8_t GetNumAudioChannels(int id);
     void JumpToTitle(bool change) { jumptotitle = change; }
-    
+    void SeekCellStart(void);
+    void SetDVDSpeed(void);
+    void SetDVDSpeed(int speed);
+
     void SetParent(NuppelVideoPlayer *p) { parent = p; }
-    
+
+
   protected:
     dvdnav_t      *dvdnav;
     unsigned char  dvdBlockWriteBuf[DVD_BLOCK_SIZE];
     unsigned char *dvdBlockReadBuf;
+    const char *dvdFilename;
     int            dvdBlockRPos;
     int            dvdBlockWPos;
     long long      pgLength;
     long long      pgcLength;
     long long      cellStart;
     long long      pgStart;
+    long long      currentpos;
     dvdnav_t      *lastNav; // This really belongs in the player.
     int32_t        part;
     int32_t        title;
@@ -145,38 +154,46 @@ class DVDRingBufferPriv
     bool           cellRepeated;
     int            buttonstreamid;
     bool           gotoCellStart;
-    /// this pts is of the first video frame decoded just after reading a menu pkt. 
-    /// menu pkt pts is not reliable
     long long      menupktpts;
     int            curAudioTrack;
     int8_t         curSubtitleTrack;
     bool           autoselectaudio;
     bool           autoselectsubtitle;
-    const char     *dvdname;
-    const char     *serialnumber;
     bool           jumptotitle;
-    bool           repeatseek;
     long long      seekpos;
     int            seekwhence;
+    QString        dvdname;
+    QString        serialnumber;
+    bool           seeking;
+    uint64_t       seektime;
+    uint           currentTime;
+    QMap<uint, uint> seekSpeedMap;
+    QMap<uint, uint> audioTrackMap;
+    QMap<uint, uint> subTrackMap;
 
     NuppelVideoPlayer *parent;
 
     QMutex menuBtnLock;
+    QMutex seekLock;
 
+    long long Seek(long long time);
     bool DrawMenuButton(uint8_t *spu_pkt, int buf_size);
     bool DVDButtonUpdate(bool b_mode);
     void ClearMenuSPUParameters(void);
     void ClearMenuButton(void);
     bool MenuButtonChanged(void);
-    uint ConvertLangCode(uint16_t code); /// converts 2char key to 3char key
+    uint ConvertLangCode(uint16_t code);
     void SelectDefaultButton(void);
     void ClearSubtitlesOSD(void);
-    
+
     int get_nibble(const uint8_t *buf, int nibble_offset);
     int decode_rle(uint8_t *bitmap, int linesize, int w, int h,
                     const uint8_t *buf, int nibble_offset, int buf_size);
     void guess_palette(uint32_t *rgba_palette,uint8_t *palette,
                         uint8_t *alpha);
+    int is_transp(const uint8_t *buf, int pitch, int n,
+                  const uint8_t *transp_color);
+    int find_smallest_bounding_rectangle(AVSubtitle *s);
 };
 
 #endif // DVD_RING_BUFFER_H_
