@@ -32,6 +32,7 @@ using namespace std;
 #define DEBUG_ATTRIB 1
 
 #define LOC QString("Channel(%1): ").arg(device)
+#define LOC_WARN QString("Channel(%1) Warning: ").arg(device)
 #define LOC_ERR QString("Channel(%1) Error: ").arg(device)
 
 static int format_to_mode(const QString& fmt, int v4l_version);
@@ -43,7 +44,7 @@ static QString mode_to_format(int mode, int v4l_version);
  */
 
 Channel::Channel(TVRec *parent, const QString &videodevice)
-    : ChannelBase(parent),
+    : DTVChannel(parent),
       device(videodevice),          videofd(-1),
       device_name(QString::null),   driver_name(QString::null),
       curList(NULL),                totalChannels(0),
@@ -57,6 +58,16 @@ Channel::~Channel(void)
     Close();
 }
 
+bool Channel::Init(QString &inputname, QString &startchannel, bool setchan)
+{
+    if (setchan)
+    {
+        SetFormat(gContext->GetSetting("TVFormat"));
+        SetDefaultFreqTable(gContext->GetSetting("FreqTable"));
+    }
+    return ChannelBase::Init(inputname, startchannel, setchan);
+}
+
 bool Channel::Open(void)
 {
 #if FAKE_VIDEO
@@ -104,18 +115,14 @@ void Channel::SetFd(int fd)
     videofd = (fd >= 0) ? fd : -1;
 }
 
-static int format_to_mode(const QString& fmt, int v4l_version)
+static int format_to_mode(const QString &fmt, int v4l_version)
 {
-    if (v4l_version==2)
+    if (2 == v4l_version)
     {
-        if (fmt == "NTSC")
-            return V4L2_STD_NTSC;
-        else if (fmt == "ATSC")
-            return V4L2_STD_ATSC_8_VSB;
-        else if (fmt == "PAL")
-            return V4L2_STD_PAL;
-        else if (fmt == "PAL-BG")
+        if (fmt == "PAL-BG")
             return V4L2_STD_PAL_BG;
+        else if (fmt == "PAL-D")
+            return V4L2_STD_PAL_D;
         else if (fmt == "PAL-DK")
             return V4L2_STD_PAL_DK;
         else if (fmt == "PAL-I")
@@ -124,6 +131,8 @@ static int format_to_mode(const QString& fmt, int v4l_version)
             return V4L2_STD_PAL_60;
         else if (fmt == "SECAM")
             return V4L2_STD_SECAM;
+        else if (fmt == "SECAM-D")
+            return V4L2_STD_SECAM_D;
         else if (fmt == "PAL-NC")
             return V4L2_STD_PAL_Nc;
         else if (fmt == "PAL-M")
@@ -132,25 +141,20 @@ static int format_to_mode(const QString& fmt, int v4l_version)
             return V4L2_STD_PAL_N;
         else if (fmt == "NTSC-JP")
             return V4L2_STD_NTSC_M_JP;
+        // generics...
+        else if (fmt.left(4) == "NTSC")
+            return V4L2_STD_NTSC;
+        else if (fmt.left(4) == "ATSC")
+            return V4L2_STD_NTSC; // We've dropped V4L ATSC support...
+        else if (fmt.left(3) == "PAL")
+            return V4L2_STD_PAL;
         return V4L2_STD_NTSC;
     }
-    if (v4l_version==1)
+    else if (1 == v4l_version)
     {
-        if (fmt == "NTSC")
-            return VIDEO_MODE_NTSC;
-        else if (fmt == "ATSC")
-            return VIDEO_MODE_ATSC;
-        else if (fmt == "PAL")
-            return VIDEO_MODE_PAL;
-        else if (fmt == "PAL-BG")
-            return VIDEO_MODE_PAL;
-        else if (fmt == "PAL-DK")
-            return VIDEO_MODE_PAL;
-        else if (fmt == "PAL-I")
-            return VIDEO_MODE_PAL;
-        else if (fmt == "PAL-60")
-            return VIDEO_MODE_PAL;
-        else if (fmt == "SECAM")
+        if (fmt == "NTSC-JP")
+            return 6;
+        else if (fmt.left(5) == "SECAM")
             return VIDEO_MODE_SECAM;
         else if (fmt == "PAL-NC")
             return 3;
@@ -158,43 +162,84 @@ static int format_to_mode(const QString& fmt, int v4l_version)
             return 4;
         else if (fmt == "PAL-N")
             return 5;
-        else if (fmt == "NTSC-JP")
-            return 6;
+        // generics...
+        else if (fmt.left(3) == "PAL")
+            return VIDEO_MODE_PAL;
+        else if (fmt.left(4) == "NTSC")
+            return VIDEO_MODE_NTSC;
+        else if (fmt.left(4) == "ATSC")
+            return VIDEO_MODE_NTSC; // We've dropped V4L ATSC support...
+        return VIDEO_MODE_NTSC;
     }
-    return VIDEO_MODE_NTSC;
+
+    VERBOSE(VB_IMPORTANT,
+            "format_to_mode() does not recognize V4L" << v4l_version);
+
+    return V4L2_STD_NTSC; // assume V4L version 2
 }
 
 static QString mode_to_format(int mode, int v4l_version)
 {
-    if (v4l_version==2)
+    if (2 == v4l_version)
     {
         if (mode == V4L2_STD_NTSC)
             return "NTSC";
-        else if (mode == V4L2_STD_ATSC_8_VSB)
-            return "ATSC";
+        else if (mode == V4L2_STD_NTSC_M_JP)
+            return "NTSC-JP";
         else if (mode == V4L2_STD_PAL)
             return "PAL";
+        else if (mode == V4L2_STD_PAL_60)
+            return "PAL-60";
         else if (mode == V4L2_STD_PAL_BG)
             return "PAL-BG";
+        else if (mode == V4L2_STD_PAL_D)
+            return "PAL-D";
         else if (mode == V4L2_STD_PAL_DK)
             return "PAL-DK";
         else if (mode == V4L2_STD_PAL_I)
             return "PAL-I";
-        else if (mode == V4L2_STD_PAL_60)
-            return "PAL-60";
-        else if (mode == V4L2_STD_SECAM)
-            return "SECAM";
-        else if (mode == V4L2_STD_PAL_Nc)
-            return "PAL-NC";
         else if (mode == V4L2_STD_PAL_M)
             return "PAL-M";
         else if (mode == V4L2_STD_PAL_N)
             return "PAL-N";
-        else if (mode == V4L2_STD_NTSC_M_JP)
-            return "NTSC-JP";
-        return "Unknown";
+        else if (mode == V4L2_STD_PAL_Nc)
+            return "PAL-NC";
+        else if (mode == V4L2_STD_SECAM)
+            return "SECAM";
+        else if (mode == V4L2_STD_SECAM_D)
+            return "SECAM-D";
+        // generic..
+        else if ((V4L2_STD_NTSC_M      == mode) ||
+                 (V4L2_STD_NTSC_443    == mode) ||
+                 (V4L2_STD_NTSC_M_KR   == mode))
+            return "NTSC";
+        else if ((V4L2_STD_PAL_B       == mode) ||
+                 (V4L2_STD_PAL_B1      == mode) ||
+                 (V4L2_STD_PAL_G       == mode) ||
+                 (V4L2_STD_PAL_H       == mode) ||
+                 (V4L2_STD_PAL_D1      == mode) ||
+                 (V4L2_STD_PAL_K       == mode))
+            return "PAL";
+        else if ((V4L2_STD_SECAM_B     == mode) ||
+                 (V4L2_STD_SECAM_DK    == mode) ||
+                 (V4L2_STD_SECAM_G     == mode) ||
+                 (V4L2_STD_SECAM_H     == mode) ||
+                 (V4L2_STD_SECAM_K     == mode) ||
+                 (V4L2_STD_SECAM_K1    == mode) ||
+                 (V4L2_STD_SECAM_L     == mode) ||
+                 (V4L2_STD_SECAM_LC    == mode))
+            return "SECAM";
+        else if ((V4L2_STD_ATSC        == mode) ||
+                 (V4L2_STD_ATSC_8_VSB  == mode) ||
+                 (V4L2_STD_ATSC_16_VSB == mode))
+        {
+            // We've dropped V4L ATSC support, but this still needs to be
+            // returned here so we will change the mode if the device is
+            // in ATSC mode already.
+            return "ATSC";
+        }
     }
-    if (v4l_version==1)
+    else if (1 == v4l_version)
     {
         if (mode == VIDEO_MODE_NTSC)
             return "NTSC";
@@ -213,6 +258,12 @@ static QString mode_to_format(int mode, int v4l_version)
         else if (mode == 6)
             return "NTSC-JP";
     }
+    else
+    {
+        VERBOSE(VB_IMPORTANT,
+                "mode_to_format() does not recognize V4L" << v4l_version);
+    }
+
     return "Unknown";
 }
 
@@ -263,7 +314,7 @@ bool Channel::InitializeInputs(void)
                 .arg(it.key()).arg((*it)->name).arg((*it)->startChanNum)
                 .arg((*it)->tuneToChannel)
                 .arg(mode_to_format((*it)->videoModeV4L1,1))
-                .arg(mode_to_format((*it)->videoModeV4L1,2)));
+                .arg(mode_to_format((*it)->videoModeV4L2,2)));
     }
 
     return valid_cnt;
@@ -343,21 +394,15 @@ int Channel::SetFreqTable(const QString &name)
 
 int Channel::GetCurrentChannelNum(const QString &channame)
 {
-    // remove part after '-' for (HDTV subchannels)
-    QString real_channame = channame;
-    int pos = channame.find('-');
-    if (pos != -1)
-        real_channame.truncate(pos);
-
     for (int i = 0; i < totalChannels; i++)
     {
-        if (real_channame == curList[i].name)
+        if (channame == curList[i].name)
             return i;
     }
-    VERBOSE(VB_IMPORTANT,
-            QString("Channel::GetCurrentChannelNum(%1): "
-                    "Failed to find Channel '%2'")
-            .arg(channame).arg(real_channame));
+
+    VERBOSE(VB_IMPORTANT, LOC_ERR +
+            QString("GetCurrentChannelNum(%1): "
+                    "Failed to find Channel").arg(channame));
 
     return -1;
 }
@@ -365,50 +410,15 @@ int Channel::GetCurrentChannelNum(const QString &channame)
 void Channel::SaveCachedPids(const pid_cache_t &pid_cache) const
 {
     int chanid = GetChanID();
-    if (chanid >= 0)
-        ChannelBase::SaveCachedPids(chanid, pid_cache);
+    if (chanid > 0)
+        DTVChannel::SaveCachedPids(chanid, pid_cache);
 }
 
 void Channel::GetCachedPids(pid_cache_t &pid_cache) const
 {
     int chanid = GetChanID();
-    if (chanid >= 0)
-        ChannelBase::GetCachedPids(chanid, pid_cache);
-}
-
-bool Channel::SetChannelByDirection(ChannelChangeDirection dir)
-{
-    if (ChannelBase::SetChannelByDirection(dir))
-        return true;
-
-    if ((CHANNEL_DIRECTION_UP != dir) && (CHANNEL_DIRECTION_DOWN != dir))
-        return false;
-
-    QString nextchan;
-    bool finished = false;
-    int chancount = 0;
-    int curchannel = GetCurrentChannelNum(curchannelname);
-    int incrDir = (CHANNEL_DIRECTION_UP == dir) ? 1 : -1;
-
-    while (!finished)
-    {
-        curchannel += incrDir;
-        curchannel = (curchannel < 0) ? totalChannels - 1 : curchannel;
-        curchannel = (curchannel > totalChannels) ? 0 : curchannel;
-        chancount++;
-
-        nextchan = curList[curchannel].name;
-        finished = SetChannelByString(nextchan);
-
-        if (chancount > totalChannels)
-        {
-            VERBOSE(VB_IMPORTANT, LOC_ERR +
-                    "Couldn't find any available channels."
-                    "\n\t\t\tYour database is most likely setup incorrectly.");
-            break;
-        }
-    }
-    return finished;
+    if (chanid > 0)
+        DTVChannel::GetCachedPids(chanid, pid_cache);
 }
 
 bool Channel::SetChannelByString(const QString &channum)
@@ -441,14 +451,18 @@ bool Channel::SetChannelByString(const QString &channum)
     if (!inputName.isEmpty())
         return ChannelBase::SwitchToInput(inputName, channum);
 
-    SetCachedATSCInfo("");
+    ClearDTVInfo();
 
     InputMap::const_iterator it = inputs.find(currentInputID);
     if (it == inputs.end())
         return false;
 
+    uint mplexid_restriction;
+    if (!IsInputAvailable(currentInputID, mplexid_restriction))
+        return false;
+
     // Fetch tuning data from the database.
-    QString tvformat, modulation, freqtable, freqid;
+    QString tvformat, modulation, freqtable, freqid, dtv_si_std;
     int finetune;
     uint64_t frequency;
     int mpeg_prog_num;
@@ -458,12 +472,15 @@ bool Channel::SetChannelByString(const QString &channum)
         (*it)->sourceid, channum,
         tvformat, modulation, freqtable, freqid,
         finetune, frequency,
-        mpeg_prog_num, atsc_major, atsc_minor, tsid, netid,
+        dtv_si_std, mpeg_prog_num, atsc_major, atsc_minor, tsid, netid,
         mplexid, commfree))
     {
         return false;
     }
 
+    if (mplexid_restriction && (mplexid != mplexid_restriction))
+        return false;
+
     // If the frequency is zeroed out, don't use it directly.
     bool ok = (frequency > 0);
 
@@ -487,18 +504,27 @@ bool Channel::SetChannelByString(const QString &channum)
     // Set NTSC, PAL, ATSC, etc.
     SetFormat(tvformat);
 
-    // Setup filters & recording picture attributes for framegrabing recorders.
-    if (pParent)
-        pParent->SetVideoFiltersForChannel(GetCurrentSourceID(), channum);
-    InitPictureAttributes();
+    // If a tuneToChannel is set make sure we're still on it
+    if (!(*it)->tuneToChannel.isEmpty() && (*it)->tuneToChannel != "Undefined")
+        TuneTo((*it)->tuneToChannel, 0);
 
     // Tune to proper frequency
     if ((*it)->externalChanger.isEmpty())
     {
-        if (isFrequency)
+        if ((*it)->name.contains("composite", false) ||
+            (*it)->name.contains("s-video", false))
         {
-            if (!Tune(frequency))
+            VERBOSE(VB_GENERAL, LOC_WARN + "You have not set "
+                    "an external channel changing"
+                    "\n\t\t\tscript for a composite or s-video "
+                    "input. Channel changing will do nothing.");
+        }
+        else if (isFrequency)
+        {
+            if (!Tune(frequency, "", (is_dtv) ? "8vsb" : "analog", dtv_si_std))
+            {
                 return false;
+            }
         }
         else
         {
@@ -512,14 +538,17 @@ bool Channel::SetChannelByString(const QString &channum)
     // Set the current channum to the new channel's channum
     curchannelname = QDeepCopy<QString>(channum);
 
+    // Setup filters & recording picture attributes for framegrabing recorders
+    // now that the new curchannelname has been established.
+    if (pParent)
+        pParent->SetVideoFiltersForChannel(GetCurrentSourceID(), channum);
+    InitPictureAttributes();
+
     // Set the major and minor channel for any additional multiplex tuning
-    if (atsc_major || atsc_minor)
-        SetCachedATSCInfo(QString("%1_%2").arg(atsc_major).arg(atsc_minor));
-    else
-        SetCachedATSCInfo(QString("%1_0").arg(channum));
+    SetDTVInfo(atsc_major, atsc_minor, netid, tsid, mpeg_prog_num);
 
     // Set this as the future start channel for this source
-    inputs[currentInputID]->startChanNum = curchannelname;
+    inputs[currentInputID]->startChanNum = QDeepCopy<QString>(curchannelname);
 
     return true;
 }
@@ -542,10 +571,16 @@ bool Channel::TuneTo(const QString &channum, int finetune)
 
     int frequency = (curList[i].freq + finetune) * 1000;
 
-    return Tune(frequency);
+    return Tune(frequency, "", "analog", "analog");
 }
 
-/** \fn Channel::Tune(uint frequency, QString inputname, QString modulation)
+bool Channel::Tune(const DTVMultiplex &tuning, QString inputname)
+{
+    return Tune(tuning.frequency - 1750000, // to visual carrier
+                inputname, tuning.modulation.toString(), tuning.sistandard);
+}
+
+/** \fn Channel::Tune(uint,QString,QString,QString)
  *  \brief Tunes to a specific frequency (Hz) on a particular input, using
  *         the specified modulation.
  *
@@ -556,10 +591,12 @@ bool Channel::TuneTo(const QString &channum, int finetune)
  *  \param inputname Name of the input (Television, Antenna 1, etc.)
  *  \param modulation "radio", "analog", or "digital"
  */
-bool Channel::Tune(uint frequency, QString inputname, QString modulation)
+bool Channel::Tune(uint frequency, QString inputname,
+                   QString modulation, QString si_std)
 {
-    VERBOSE(VB_CHANNEL, QString("Channel(%1)::Tune(%2, %3, %4)")
-            .arg(device).arg(frequency).arg(inputname).arg(modulation));
+    VERBOSE(VB_CHANNEL, LOC + QString("Tune(%1, %2, %3, %4)")
+            .arg(frequency).arg(inputname).arg(modulation).arg(si_std));
+
     int ioctlval = 0;
 
     if (modulation == "8vsb")
@@ -657,6 +694,8 @@ bool Channel::Tune(uint frequency, QString inputname, QString modulation)
         return false;
     }
 
+    SetSIStandard(si_std);
+
     return true;
 }
 
@@ -697,77 +736,29 @@ bool Channel::Retune(void)
     return false;
 }
 
-bool Channel::IsTuned() const
+// documented in dtvchannel.h
+bool Channel::TuneMultiplex(uint mplexid, QString inputname)
 {
-    if (usingv4l2)
+    VERBOSE(VB_CHANNEL, LOC + QString("TuneMultiplex(%1)").arg(mplexid));
+
+    QString  modulation;
+    QString  si_std;
+    uint64_t frequency;
+    uint     transportid;
+    uint     dvb_networkid;
+
+    if (!ChannelUtil::GetTuningParams(
+            mplexid, modulation, frequency,
+            transportid, dvb_networkid, si_std))
     {
-        struct v4l2_tuner tuner;
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "TuneMultiplex(): " +
+                QString("Could not find tuning parameters for multiplex %1.")
+                .arg(mplexid));
 
-        memset(&tuner,0,sizeof(tuner));
-        if (-1 == ::ioctl(videofd,VIDIOC_G_TUNER,&tuner,0))
-            return false;
-        return tuner.signal ? true : false;
-    }
-    else 
-    {
-        struct video_tuner tuner;
-        memset(&tuner,0,sizeof(tuner));
-        if (-1 == ::ioctl(videofd,VIDIOCGTUNER,&tuner,0))
-             return false;
-        return tuner.signal ? true : false;
-    }
-}
-
-/** \fn Channel::TuneMultiplex(uint mplexid)
- *  \brief To be used by the siscan
- *
- *   mplexid is how the db indexes each transport
- */
-bool Channel::TuneMultiplex(uint mplexid)
-{
-    VERBOSE(VB_CHANNEL, QString("Channel(%1)::TuneMultiplex(%2)")
-            .arg(device).arg(mplexid));
-
-    MSqlQuery query(MSqlQuery::InitCon());
-
-    int cardid = GetCardID();
-    if (cardid < 0)
-        return false;
-
-    // Query for our tuning params
-    QString thequery(
-        "SELECT frequency, inputname, modulation "
-        "FROM dtv_multiplex, cardinput, capturecard "
-        "WHERE dtv_multiplex.sourceid = cardinput.sourceid AND "
-        "      cardinput.cardid = capturecard.cardid AND ");
-
-    thequery += QString("mplexid = '%1' AND cardinput.cardid = '%2'")
-        .arg(mplexid).arg(cardid);
-
-    query.prepare(thequery);
-    if (!query.exec() || !query.isActive())
-    {
-        MythContext::DBError(
-            QString("Channel(%1)::TuneMultiplex(): Error, could not find "
-                    "tuning parameters for transport %1.")
-            .arg(device).arg(mplexid), query);
         return false;
     }
-    if (query.size() <= 0)
-    {
-        VERBOSE(VB_IMPORTANT, QString(
-                    "Channel(%1)::TuneMultiplex(): Error, could not find "
-                    "tuning parameters for transport %1.")
-                .arg(device).arg(mplexid));
-        return false;
-    }
-    query.next();
 
-    uint    frequency  = query.value(0).toInt();
-    QString inputname  = query.value(1).toString();
-    QString modulation = query.value(2).toString();
-
-    if (!Tune(frequency, inputname, modulation))
+    if (!Tune(frequency, inputname, modulation, si_std))
         return false;
 
     return true;
@@ -853,9 +844,6 @@ bool Channel::SetInputAndFormat(int inputNum, QString newFmt)
             VERBOSE(VB_IMPORTANT, LOC_ERR + msg +
                     "\n\t\t\twhile setting format (v4l v2)" + ENO);
 
-            // Fall through to try v4l version 1, pcHDTV 1.4 (for HD-2000)
-            // drivers don't work with VIDIOC_S_STD ioctl.
-            usingv4l1 = true;
             ok = false;
         }
 
@@ -917,6 +905,10 @@ bool Channel::SwitchToInput(int inputnum, bool setstarting)
             .arg(device).arg(inputnum)
             .arg(setstarting ? channum : QString("")));
 
+    uint mplexid_restriction;
+    if (!IsInputAvailable(inputnum, mplexid_restriction))
+        return false;
+
     QString newFmt = mode_to_format((*it)->videoModeV4L2, 2);
 
     // If we are setting a channel, get its video mode...
@@ -945,7 +937,7 @@ bool Channel::SwitchToInput(int inputnum, bool setstarting)
     currentInputID = inputnum;
     curchannelname    = ""; // this will be set by SetChannelByString
 
-    if (setstarting && !tuneFreqId.isEmpty() && tuneFreqId != "Undefined")
+    if (!tuneFreqId.isEmpty() && tuneFreqId != "Undefined")
         ok = TuneTo(tuneFreqId, 0);
 
     if (!ok)
@@ -1010,7 +1002,7 @@ bool Channel::InitPictureAttribute(const QString db_col_name)
     int cfield = ChannelUtil::GetChannelValueInt(
         db_col_name, GetCurrentSourceID(), curchannelname);
     int sfield = CardUtil::GetValueInt(
-        db_col_name, GetCardID(), GetCurrentSourceID());
+        db_col_name, GetCardID());
 
     if ((cfield == -1) || (sfield == -1))
         return false;
@@ -1064,8 +1056,9 @@ bool Channel::InitPictureAttribute(const QString db_col_name)
         ctrl.value = max(value1, (int)qctrl.minimum);
 
 #if DEBUG_ATTRIB
-        VERBOSE(VB_CHANNEL, loc + QString("\n\t\t\t[%1,%2] dflt(%3, %4, %5)")
-                .arg(qctrl.minimum, 5).arg(qctrl.maximum, 5)
+        VERBOSE(VB_CHANNEL, loc + QString(" %1\n\t\t\t"
+                                          "[%2,%3] dflt(%4, %5, %6)")
+                .arg(value0).arg(qctrl.minimum, 5).arg(qctrl.maximum, 5)
                 .arg(qctrl.default_value, 5).arg(dfl, 4, 'f', 2)
                 .arg(norm_dfl));
 #endif
@@ -1112,12 +1105,16 @@ bool Channel::InitPictureAttributes(void)
             InitPictureAttribute("hue"));
 }
 
-int Channel::GetPictureAttribute(const QString db_col_name) const
+int Channel::GetPictureAttribute(PictureAttribute attr) const
 {
+    QString db_col_name = toDBString(attr);
+    if (db_col_name.isEmpty())
+        return -1;
+
     int cfield = ChannelUtil::GetChannelValueInt(
         db_col_name, GetCurrentSourceID(), curchannelname);
     int sfield = CardUtil::GetValueInt(
-        db_col_name, GetCardID(), GetCurrentSourceID());
+        db_col_name, GetCardID());
     int dfield = 0;
 
     if (pict_attr_default.find(db_col_name) != pict_attr_default.end())
@@ -1157,7 +1154,8 @@ static int get_v4l2_attribute_value(int videofd, int v4l2_attrib)
         return -1;
     }
 
-    return (int)(65535.0 / (qctrl.maximum - qctrl.minimum) * ctrl.value);
+    float mult = 65535.0 / (qctrl.maximum - qctrl.minimum);
+    return min(max((int)(mult * (ctrl.value - qctrl.minimum)), 0), 65525);
 }
 
 static int get_v4l1_attribute_value(int videofd, int v4l2_attrib)
@@ -1257,11 +1255,15 @@ static int set_attribute_value(bool usingv4l2, int videofd,
 }
 
 int Channel::ChangePictureAttribute(
-    int type, const QString db_col_name, bool up)
+    PictureAdjustType type, PictureAttribute attr, bool up)
 {
     if (!pParent || is_dtv)
         return -1;
 
+    QString db_col_name = toDBString(attr);
+    if (db_col_name.isEmpty())
+        return -1;
+
     int v4l2_attrib = get_v4l2_attribute(db_col_name);
     if (v4l2_attrib == -1)
         return -1;
@@ -1271,7 +1273,7 @@ int Channel::ChangePictureAttribute(
     if (get_attribute_value(usingv4l2, videofd, v4l2_attrib) < 0)
         return -1;
 
-    int old_value = GetPictureAttribute(db_col_name);
+    int old_value = GetPictureAttribute(attr);
     int new_value = old_value + ((up) ? 655 : -655);
 
     // make sure we are within bounds (wrap around for hue)
@@ -1305,7 +1307,7 @@ int Channel::ChangePictureAttribute(
     else if (kAdjustingPicture_Recording == type)
     {
         int adj_value = CardUtil::GetValueInt(
-            db_col_name, GetCardID(), GetCurrentSourceID());
+            db_col_name, GetCardID());
 
         int tmp = new_value - old_value + adj_value;
         tmp = (tmp < 0)      ? tmp + 0x10000 : tmp;
