@@ -7,6 +7,9 @@
 #include <algorithm>
 using namespace std;
 
+// Qt headers
+#include <qdeepcopy.h>
+
 // MythTV includes
 #include "eit.h"
 #include "eithelper.h"
@@ -15,10 +18,14 @@ using namespace std;
 #include "mythdbcon.h"
 #include "atsctables.h"
 #include "dvbtables.h"
+#include "premieretables.h"
 #include "dishdescriptors.h"
+#include "premieredescriptors.h"
 #include "util.h"
+#include "programinfo.h" // for subtitle types and audio and video properties
 
 const uint EITHelper::kChunkSize = 20;
+EITCache *EITHelper::eitcache = new EITCache();
 
 static uint get_chan_id_from_db(uint sourceid,
                                 uint atscmajor, uint atscminor);
@@ -31,7 +38,7 @@ static int calc_eit_utc_offset(void);
 #define LOC_ERR QString("EITHelper, Error: ")
 
 EITHelper::EITHelper() :
-    eitfixup(new EITFixUp()), eitcache(new EITCache()),
+    eitfixup(new EITFixUp()),
     gps_offset(-1 * GPS_LEAP_SECONDS),          utc_offset(0),
     sourceid(0)
 {
@@ -44,7 +51,7 @@ EITHelper::EITHelper() :
     int hours   = diff / (60 * 60);
     int minutes = ((diff) / 60) % 60;
     int seconds = diff % 60;
-    VERBOSE(VB_IMPORTANT, LOC + QString("localtime offset %1%2:%3%4:%5%6 ")
+    VERBOSE(VB_EIT, LOC + QString("localtime offset %1%2:%3%4:%5%6 ")
             .arg((sign < 0) ? "-" : "")
             .arg(hours).arg(minutes/10).arg(minutes%10)
             .arg(seconds/10).arg(seconds%10));
@@ -57,7 +64,6 @@ EITHelper::~EITHelper()
         delete db_events.dequeue();
 
     delete eitfixup;
-    delete eitcache;
 }
 
 uint EITHelper::GetListSize(void) const
@@ -69,7 +75,6 @@ uint EITHelper::GetListSize(void) const
 /** \fn EITHelper::ProcessEvents(void)
  *  \brief Inserts events in EIT list.
  *
- *  \param mplexid multiplex we are inserting events for.
  *  \return Returns number of events inserted into DB.
  */
 uint EITHelper::ProcessEvents(void)
@@ -211,6 +216,88 @@ void EITHelper::AddETT(uint atsc_major, uint atsc_minor,
     }
 }
 
+static void parse_dvb_event_descriptors(desc_list_t list, uint fix,
+                                        QMap<uint,uint> languagePreferences,
+                                        QString &title, QString &subtitle,
+                                        QString &description)
+{
+    const unsigned char *bestShortEvent =
+        MPEGDescriptor::FindBestMatch(
+            list, DescriptorID::short_event, languagePreferences);
+
+    unsigned char enc_1[3]  = { 0x10, 0x00, 0x01 };
+    unsigned char enc_15[3] = { 0x10, 0x00, 0x0f };
+    int enc_len = 0;
+    const unsigned char *enc = NULL;
+
+    // Is this BellExpressVU EIT (Canada) ?
+    // Use an encoding override of ISO 8859-1 (Latin1)
+    if (fix & EITFixUp::kEFixForceISO8859_1)
+    {
+        enc = enc_1;
+        enc_len = sizeof(enc_1);
+    }
+
+    // Is this broken DVB provider in Western Europe?
+    // Use an encoding override of ISO 8859-15 (Latin6)
+    if (fix & EITFixUp::kEFixForceISO8859_15)
+    {
+        enc = enc_15;
+        enc_len = sizeof(enc_15);
+    }
+
+    if (bestShortEvent)
+    {
+        ShortEventDescriptor sed(bestShortEvent);
+        if (enc)
+        {
+            title    = sed.EventName(enc, enc_len);
+            subtitle = sed.Text(enc, enc_len);
+        }
+        else
+        {
+            title    = sed.EventName();
+            subtitle = sed.Text();
+        }
+    }
+
+    vector<const unsigned char*> bestExtendedEvents =
+        MPEGDescriptor::FindBestMatches(
+            list, DescriptorID::extended_event, languagePreferences);
+
+    description = "";
+    for (uint j = 0; j < bestExtendedEvents.size(); j++)
+    {
+        if (!bestExtendedEvents[j])
+        {
+            description = "";
+            break;
+        }
+
+        ExtendedEventDescriptor eed(bestExtendedEvents[j]);
+        if (enc)
+            description += eed.Text(enc, enc_len);
+        else
+            description += eed.Text();
+    }
+}
+
+static inline void parse_dvb_component_descriptors(desc_list_t list,
+                                                   unsigned char &subtitle_type,
+                                                   unsigned char &audio_properties,
+                                                   unsigned char &video_properties)
+{
+    desc_list_t components =
+        MPEGDescriptor::FindAll(list, DescriptorID::component);
+    for (uint j = 0; j < components.size(); j++)
+    {
+        ComponentDescriptor component(components[j]);
+        video_properties |= component.VideoProperties();
+        audio_properties |= component.AudioProperties();
+        subtitle_type    |= component.SubtitleType();
+    }
+}
+
 void EITHelper::AddEIT(const DVBEventInformationTable *eit)
 {
     uint descCompression = (eit->TableID() > 0x80) ? 2 : 1;
@@ -223,16 +310,18 @@ void EITHelper::AddEIT(const DVBEventInformationTable *eit)
 		 (uint64_t)eit->ServiceID()];
     fix |= EITFixUp::kFixGenericDVB;
 
-    uint networkid = eit->OriginalNetworkID();
-    uint tsid      = eit->TSID();
-    uint serviceid = eit->ServiceID();
+    uint chanid = GetChanID(eit->ServiceID(), eit->OriginalNetworkID(),
+                            eit->TSID());
+    if (!chanid)
+        return;
+
     uint tableid   = eit->TableID();
     uint version   = eit->Version();
     for (uint i = 0; i < eit->EventCount(); i++)
     {
         // Skip event if we have already processed it before...
-        if (!eitcache->IsNewEIT(networkid, tsid, serviceid, tableid, version,
-                                eit->EventID(i), eit->EndTimeUnixUTC(i)))
+        if (!eitcache->IsNewEIT(chanid, tableid, version, eit->EventID(i),
+                              eit->EndTimeUnixUTC(i)))
         {
             continue;
         }
@@ -242,7 +331,7 @@ void EITHelper::AddEIT(const DVBEventInformationTable *eit)
         QString description   = QString::null;
         QString category      = QString::null;
         MythCategoryType category_type = kCategoryNone;
-        bool hdtv = false, stereo = false, subtitled = false;
+        unsigned char subtitle_type=0, audio_props=0, video_props=0;
 
         // Parse descriptors
         desc_list_t list = MPEGDescriptor::Parse(
@@ -269,59 +358,12 @@ void EITHelper::AddEIT(const DVBEventInformationTable *eit)
         }
         else
         {
-            const unsigned char *bestShortEvent =
-                MPEGDescriptor::FindBestMatch(
-                    list, DescriptorID::short_event, languagePreferences);
-
-            unsigned char enc_ch[1] = { 0x05 };
-            const unsigned char *enc =
-                (fix & EITFixUp::kEFixPro7Sat) ? enc_ch : NULL;
-
-            if (bestShortEvent)
-            {
-                ShortEventDescriptor sed(bestShortEvent);
-                if (enc)
-                {
-                    title    = sed.EventName(enc, 1);
-                    subtitle = sed.Text(enc, 1);
-                }
-                else
-                {
-                    title    = sed.EventName();
-                    subtitle = sed.Text();
-                }
-            }
-
-            vector<const unsigned char*> bestExtendedEvents =
-                MPEGDescriptor::FindBestMatches(
-                    list, DescriptorID::extended_event, languagePreferences);
-
-            description = "";
-            for (uint j = 0; j < bestExtendedEvents.size(); j++)
-            {
-                if (!bestExtendedEvents[j])
-                {
-                    description = "";
-                    break;
-                }
-
-                ExtendedEventDescriptor eed(bestExtendedEvents[j]);
-                if (enc)
-                    description += eed.Text(enc, 1);
-                else
-                    description += eed.Text();
-            }
+            parse_dvb_event_descriptors(list, fix, languagePreferences,
+                                        title, subtitle, description);
         }
 
-        desc_list_t components =
-            MPEGDescriptor::FindAll(list, DescriptorID::component);
-        for (uint j = 0; j < components.size(); j++)
-        {
-            ComponentDescriptor component(components[j]);
-            hdtv      |= component.IsHDTV();
-            stereo    |= component.IsStereo();
-            subtitled |= component.IsReallySubtitled();
-        }
+        parse_dvb_component_descriptors(list, subtitle_type, audio_props,
+                                        video_props);
 
         const unsigned char *content_data =
             MPEGDescriptor::Find(list, DescriptorID::content);
@@ -332,11 +374,23 @@ void EITHelper::AddEIT(const DVBEventInformationTable *eit)
             category_type = content.GetMythCategory(0);
         }
 
-        uint chanid = GetChanID(
-            eit->ServiceID(), eit->OriginalNetworkID(), eit->TSID());
-
-        if (!chanid)
-            continue;
+        desc_list_t contentIds =
+            MPEGDescriptor::FindAll(list, DescriptorID::dvb_content_identifier);
+        QString programId = "", seriesId = "";
+        for (uint j = 0; j < contentIds.size(); j++)
+        {
+            DVBContentIdentifierDescriptor desc(contentIds[j]);
+            if (desc.ContentEncoding() == 0)
+            {
+                // The CRID is a URI.  It could contain UTF8 sequences encoded
+                // as %XX but there's no advantage in decoding them.
+                // The BBC currently uses private types 0x31 and 0x32.
+                if (desc.ContentType() == 0x01 || desc.ContentType() == 0x31)
+                    programId = desc.ContentId();
+                else if (desc.ContentType() == 0x02 || desc.ContentType() == 0x32)
+                    seriesId = desc.ContentId();
+            }
+        }
 
         QDateTime starttime = MythUTCToLocal(eit->StartTimeUTC(i));
         EITFixUp::TimeFix(starttime);
@@ -346,17 +400,128 @@ void EITHelper::AddEIT(const DVBEventInformationTable *eit)
                                      title,     subtitle,      description,
                                      category,  category_type,
                                      starttime, endtime,       fix,
-                                     false,     subtitled,
-                                     stereo,    hdtv);
+                                     subtitle_type,
+                                     audio_props,
+                                     video_props,
+                                     seriesId,  programId);
         db_events.enqueue(event);
     }
 }
 
-void EITHelper::PruneCache(uint timestamp)
+// This function gets special eit data from the german provider Premiere
+// for the option channels Premiere Sport and Premiere Direkt
+void EITHelper::AddEIT(const PremiereContentInformationTable *cit)
+{
+    // set fixup for Premiere
+    uint fix = fixup[133 << 16];
+    fix |= EITFixUp::kFixGenericDVB;
+
+    QString title         = QString::null;
+    QString subtitle      = QString::null;
+    QString description   = QString::null;
+    QString category      = QString::null;
+    MythCategoryType category_type = kCategoryNone;
+    unsigned char subtitle_type=0, audio_props=0, video_props=0;
+
+    // Parse descriptors
+    desc_list_t list = MPEGDescriptor::Parse(
+        cit->Descriptors(), cit->DescriptorsLength());
+
+    parse_dvb_event_descriptors(list, fix, languagePreferences,
+                                title, subtitle, description);
+
+    parse_dvb_component_descriptors(list, subtitle_type, audio_props,
+                                    video_props);
+
+    const unsigned char *content_data =
+        MPEGDescriptor::Find(list, DescriptorID::content);
+    if (content_data)
+    {
+        ContentDescriptor content(content_data);
+        // fix events without real content data
+        if (content.Nibble(0)==0x00){
+            if(content.UserNibble(0)==0x1)
+            {
+                category_type = kCategoryMovie;
+            }
+            else if(content.UserNibble(0)==0x0)
+            {
+                category_type = kCategorySports;
+                category = QObject::tr("Sports");
+            }
+        }
+        else
+        {
+            category_type = content.GetMythCategory(0);
+            category      = content.GetDescription(0);
+        }
+    }
+
+    uint tableid   = cit->TableID();
+    uint version   = cit->Version();
+    uint contentid = cit->ContentID();
+    // fake endtime
+    uint endtime   = QDateTime::currentDateTime().addDays(1).toTime_t();
+
+    // Find Transmissions
+    desc_list_t transmissions =
+        MPEGDescriptor::FindAll(list,
+                                DescriptorID::premiere_content_transmission);
+    for(uint j=0; j< transmissions.size(); j++)
+    {
+        PremiereContentTransmissionDescriptor transmission(transmissions[j]);
+        uint networkid = transmission.OriginalNetworkID();
+        uint tsid      = transmission.TSID();
+        uint serviceid = transmission.ServiceID();
+
+        uint chanid = GetChanID(serviceid, networkid, tsid);
+
+        if (!chanid)
+        {
+            VERBOSE(VB_EIT, LOC +
+                    QString("Premiere EIT for NIT %1, TID %2, SID %3, count %4, "
+                            "title: %5. Channel not found!")
+                    .arg(networkid).arg(tsid).arg(serviceid)
+                    .arg(transmission.TransmissionCount()).arg(title));
+            continue;
+        }
+
+        // Skip event if we have already processed it before...
+        if (!eitcache->IsNewEIT(chanid, tableid, version, contentid, endtime))
+        {
+            continue;
+        }
+
+        for (uint k=0; k<transmission.TransmissionCount(); ++k)
+        {
+            QDateTime starttime = transmission.StartTimeUTC(k);
+            EITFixUp::TimeFix(starttime);
+            QDateTime endtime   = starttime.addSecs(cit->DurationInSeconds());
+
+            DBEvent *event = new DBEvent(chanid,
+                                         title,     subtitle,      description,
+                                         category,  category_type,
+                                         starttime, endtime,       fix,
+                                         subtitle_type,
+                                         audio_props,
+                                         video_props,
+                                         "",  "");
+            db_events.enqueue(event);
+        }
+    }
+}
+
+
+void EITHelper::PruneEITCache(uint timestamp)
 {
     eitcache->PruneOldEntries(timestamp);
 }
 
+void EITHelper::WriteEITCache(void)
+{
+    eitcache->WriteToDB();
+}
+
 //////////////////////////////////////////////////////////////////////
 // private methods and functions below this line                    //
 //////////////////////////////////////////////////////////////////////
@@ -390,8 +555,11 @@ void EITHelper::CompleteEvent(uint atsc_major, uint atsc_minor,
     QDateTime endtime = starttime.addSecs(event.length);
 
     desc_list_t list = MPEGDescriptor::Parse(event.desc, event.desc_length);
-    bool captioned = MPEGDescriptor::Find(list, DescriptorID::caption_service);
-    bool stereo = false;
+    unsigned char subtitle_type =
+        MPEGDescriptor::Find(list, DescriptorID::caption_service) ?
+        SUB_HARDHEAR : SUB_UNKNOWN;
+    unsigned char audio_properties = AUD_UNKNOWN;
+    unsigned char video_properties = VID_UNKNOWN;
 
     uint atsc_key = (atsc_major << 16) | atsc_minor;
 
@@ -399,7 +567,8 @@ void EITHelper::CompleteEvent(uint atsc_major, uint atsc_minor,
     db_events.enqueue(new DBEvent(chanid, QDeepCopy<QString>(event.title),
                                   QDeepCopy<QString>(ett),
                                   starttime, endtime,
-                                  fixup[atsc_key], captioned, stereo));
+                                  fixup[atsc_key], subtitle_type,
+                                  audio_properties, video_properties));
 }
 
 uint EITHelper::GetChanID(uint atsc_major, uint atsc_minor)
@@ -504,23 +673,29 @@ static uint get_chan_id_from_db(uint sourceid, uint serviceid,
 
 static void init_fixup(QMap<uint64_t,uint> &fix)
 {
+    ///////////////////////////////////////////////////////////////////////////
+    // Fixups to make EIT provided listings more useful
     // transport_id<<32 | netword_id<<16 | service_id
-    fix[  256 << 16] = EITFixUp::kFixBell;
-    fix[  257 << 16] = EITFixUp::kFixBell;
-    fix[ 4100 << 16] = EITFixUp::kFixBell;
-    fix[ 4101 << 16] = EITFixUp::kFixBell;
-    fix[ 4102 << 16] = EITFixUp::kFixBell;
-    fix[ 4103 << 16] = EITFixUp::kFixBell;
-    fix[ 4104 << 16] = EITFixUp::kFixBell;
-    fix[ 4105 << 16] = EITFixUp::kFixBell;
-    fix[ 4106 << 16] = EITFixUp::kFixBell;
-    fix[ 4107 << 16] = EITFixUp::kFixBell;
-    fix[ 4097 << 16] = EITFixUp::kFixBell;
-    fix[ 4098 << 16] = EITFixUp::kFixBell;
 
-    fix[ 9018 << 16] = EITFixUp::kFixUK;
+    // Bell Express VU Canada
+    fix[  256U << 16] = EITFixUp::kFixBell;
+    fix[  257U << 16] = EITFixUp::kFixBell;
+    fix[ 4100U << 16] = EITFixUp::kFixBell;
+    fix[ 4101U << 16] = EITFixUp::kFixBell;
+    fix[ 4102U << 16] = EITFixUp::kFixBell;
+    fix[ 4103U << 16] = EITFixUp::kFixBell;
+    fix[ 4104U << 16] = EITFixUp::kFixBell;
+    fix[ 4105U << 16] = EITFixUp::kFixBell;
+    fix[ 4106U << 16] = EITFixUp::kFixBell;
+    fix[ 4107U << 16] = EITFixUp::kFixBell;
+    fix[ 4097U << 16] = EITFixUp::kFixBell;
+    fix[ 4098U << 16] = EITFixUp::kFixBell;
 
-    fix[40999U << 16] = EITFixUp::kFixComHem;
+    // United Kingdom
+    fix[ 9018U << 16] = EITFixUp::kFixUK;
+
+    // ComHem Sweeden
+    fix[40999U << 16       ] = EITFixUp::kFixComHem;
     fix[40999U << 16 | 1070] = EITFixUp::kFixSubtitle;
     fix[40999U << 16 | 1308] = EITFixUp::kFixSubtitle;
     fix[40999U << 16 | 1041] = EITFixUp::kFixSubtitle;
@@ -532,18 +707,67 @@ static void init_fixup(QMap<uint64_t,uint> &fix)
     fix[40999U << 16 | 1068] = EITFixUp::kFixSubtitle;
     fix[40999U << 16 | 1069] = EITFixUp::kFixSubtitle;
 
-    fix[ 4096 << 16] = EITFixUp::kFixAUStar;
-    fix[ 4096 << 16] = EITFixUp::kFixAUStar;
+    // Australia
+    fix[ 4096U << 16] = EITFixUp::kFixAUStar;
+    fix[ 4096U << 16] = EITFixUp::kFixAUStar;
 
-    // DVB-C germany: Kabel Deutschland encoding fixes
-    fix[   112LL << 32 | 61441U << 16] = EITFixUp::kEFixPro7Sat;
-    fix[ 10000LL << 32 | 61441U << 16] = EITFixUp::kEFixPro7Sat;
-    fix[ 10001LL << 32 | 61441U << 16] = EITFixUp::kEFixPro7Sat;
-    fix[ 10002LL << 32 | 61441U << 16] = EITFixUp::kEFixPro7Sat;
-    fix[ 10003LL << 32 | 61441U << 16] = EITFixUp::kEFixPro7Sat;
-    fix[ 10005LL << 32 | 61441U << 16] = EITFixUp::kEFixPro7Sat;
-    fix[ 10006LL << 32 | 61441U << 16] = EITFixUp::kEFixPro7Sat;
-    fix[ 10009LL << 32 | 61441U << 16] = EITFixUp::kEFixPro7Sat;
+    // MultiChoice Africa
+    fix[ 6144U << 16] = EITFixUp::kFixMCA;
+
+    // RTL Subtitle parsing
+    fix[      1089LL << 32 |     1  << 16] = // DVB-S
+        fix[   773LL << 32 |  8468U << 16] = // DVB-T Berlin/Brandenburg
+        fix[  2819LL << 32 |  8468U << 16] = // DVB-T Niedersachsen + Bremen
+        fix[  8706LL << 32 |  8468U << 16] = // DVB-T NRW
+        fix[ 12801LL << 32 |  8468U << 16] = // DVB-T Bayern
+        EITFixUp::kFixRTL;
+
+    // Premiere EIT processing
+    fix[   1LL << 32 |  133 << 16] = EITFixUp::kFixPremiere;
+    fix[   2LL << 32 |  133 << 16] = EITFixUp::kFixPremiere;
+    fix[   3LL << 32 |  133 << 16] = EITFixUp::kFixPremiere;
+    fix[   4LL << 32 |  133 << 16] = EITFixUp::kFixPremiere;
+    fix[   5LL << 32 |  133 << 16] = EITFixUp::kFixPremiere;
+    fix[   6LL << 32 |  133 << 16] = EITFixUp::kFixPremiere;
+    fix[  17LL << 32 |  133 << 16] = EITFixUp::kFixPremiere;
+    // Mark Premiere HD and Discovery HD as HDTV
+    fix[   6LL << 32 |  133 << 16 | 129] = EITFixUp::kFixHDTV;
+    fix[   6LL << 32 |  133 << 16 | 130] = EITFixUp::kFixHDTV;
+    
+    // Netherlands
+    fix[ 1000U << 16] = EITFixUp::kFixNL;
+
+    ///////////////////////////////////////////////////////////////////////////
+    // Special Early fixups for providers that break DVB EIT spec.
+    // transport_id<<32 | netword_id<<16 | service_id
+
+    // Bell Express VU Canada
+    fix[ 256U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[ 257U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4100U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4101U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4102U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4103U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4104U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4105U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4106U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4107U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4097U << 16] |= EITFixUp::kEFixForceISO8859_1;
+    fix[4098U << 16] |= EITFixUp::kEFixForceISO8859_1;
+
+    //DVB-T Germany Berlin HSE/MonA TV
+    fix[  772LL << 32 | 8468 << 16 | 16387] = EITFixUp::kEFixForceISO8859_15;
+    //DVB-T Germany Ruhrgebiet Tele 5
+    fix[ 8707LL << 32 | 8468 << 16 | 16413] = EITFixUp::kEFixForceISO8859_15;
+
+    // DVB-C Kabel Deutschland encoding fixes Germany
+    fix[   112LL << 32 | 61441U << 16] = EITFixUp::kEFixForceISO8859_15;
+    fix[ 10000LL << 32 | 61441U << 16] = EITFixUp::kEFixForceISO8859_15;
+    fix[ 10001LL << 32 | 61441U << 16] = EITFixUp::kEFixForceISO8859_15;
+    fix[ 10002LL << 32 | 61441U << 16] = EITFixUp::kEFixForceISO8859_15;
+    fix[ 10003LL << 32 | 61441U << 16] = EITFixUp::kEFixForceISO8859_15;
+    fix[ 10006LL << 32 | 61441U << 16] = EITFixUp::kEFixForceISO8859_15;
+    fix[ 10009LL << 32 | 61441U << 16] = EITFixUp::kEFixForceISO8859_15;
     // On transport 10004 only DMAX needs no fixing:
     fix[    10004LL<<32 | 61441U << 16 | 50403] = // BBC World Service
         fix[10004LL<<32 | 61441U << 16 | 53101] = // BBC Prime (engl)
@@ -562,53 +786,58 @@ static void init_fixup(QMap<uint64_t,uint> &fix)
         fix[10004LL<<32 | 61441U << 16 | 53513] = // Playhouse Disney (engl)
         fix[10004LL<<32 | 61441U << 16 | 53618] = // K1010
         fix[10004LL<<32 | 61441U << 16 | 53619] = // GemsTV
-        EITFixUp::kEFixPro7Sat;
+        EITFixUp::kEFixForceISO8859_15;
+    // On transport 10005 QVC and Giga Digital  needs no fixing:
+    fix[    10005LL<<32 | 61441U << 16 | 50104] = // E! Entertainment
+        fix[10005LL<<32 | 61441U << 16 | 50107] = // 13th Street (KD)
+        fix[10005LL<<32 | 61441U << 16 | 50301] = // ESPN Classic
+        fix[10005LL<<32 | 61441U << 16 | 50302] = // VH1 Classic
+        fix[10005LL<<32 | 61441U << 16 | 50303] = // Wein TV
+        fix[10005LL<<32 | 61441U << 16 | 50304] = // AXN
+        fix[10005LL<<32 | 61441U << 16 | 50305] = // Silverline
+        fix[10005LL<<32 | 61441U << 16 | 50306] = // NASN
+        fix[10005LL<<32 | 61441U << 16 | 50307] = // Disney Toon
+        fix[10005LL<<32 | 61441U << 16 | 53105] = // NASN (engl)
+        fix[10005LL<<32 | 61441U << 16 | 53115] = // VH1 Classic (engl)
+        fix[10005LL<<32 | 61441U << 16 | 53405] = // ESPN Classic (engl)
+        fix[10005LL<<32 | 61441U << 16 | 53402] = // AXN (engl)
+        fix[10005LL<<32 | 61441U << 16 | 53613] = // CNN (engl)
+        fix[10005LL<<32 | 61441U << 16 | 53516] = // Voyages Television
+        fix[10005LL<<32 | 61441U << 16 | 53611] = // Der Schmuckkanal
+        fix[10005LL<<32 | 61441U << 16 | 53104] = // Jukebox
+        EITFixUp::kEFixForceISO8859_15;
     // On transport 10007 only following channels need fixing:
-    fix[    10007LL<<32| 61441U << 16 | 53605] = // terranova
-        fix[10007LL<<32| 61441U << 16 | 53607] = // Eurosport
+    fix[    10007LL<<32| 61441U << 16 | 53607] = // Eurosport
         fix[10007LL<<32| 61441U << 16 | 53608] = // Das Vierte
         fix[10007LL<<32| 61441U << 16 | 53609] = // Viva
         fix[10007LL<<32| 61441U << 16 | 53628] = // COMEDY CENTRAL
-        EITFixUp::kEFixPro7Sat;
+        EITFixUp::kEFixForceISO8859_15;
     // On transport 10008 only following channels need fixing:
     fix[    10008LL<<32 | 61441U << 16 | 53002] = // Tele 5
-        fix[10008LL<<32 | 61441U << 16 | 53624] = // DSF
         fix[10008LL<<32 | 61441U << 16 | 53630] = // HSE24
-        EITFixUp::kEFixPro7Sat;
+        EITFixUp::kEFixForceISO8859_15;
 
-    fix[ 774LL << 32 | 8468 << 16 | 16392] = EITFixUp::kEFixPro7Sat; //DVB-T Berlin dsf
-    fix[ 772LL << 32 | 8468 << 16 | 16387] = EITFixUp::kEFixPro7Sat; //DVB-T Berlin HSE/MonA TV
-    fix[8707LL << 32 | 8468 << 16 | 16413] = EITFixUp::kEFixPro7Sat; //DVB-T Ruhrgebiet Tele 5
-    fix[1082LL << 32 |    1 << 16 | 20001] = EITFixUp::kEFixPro7Sat; //DVB-S Pro7 Swiss
-    fix[1082LL << 32 |    1 << 16 | 20002] = EITFixUp::kEFixPro7Sat; //DVB-S Pro7 Austria
-    fix[1082LL << 32 |    1 << 16 | 20003] = EITFixUp::kEFixPro7Sat; //DVB-S Kabel1 Swiss
-    fix[1082LL << 32 |    1 << 16 | 20004] = EITFixUp::kEFixPro7Sat; //DVB-S Kabel1 Austria
-    fix[1082LL << 32 |    1 << 16 | 20005] = EITFixUp::kEFixPro7Sat; //DVB-S Sat.1 Austria
+    // DVB-S Astra 19.2E DMAX Germany
+    fix[  1113LL << 32 | 1 << 16 | 12602] = EITFixUp::kEFixForceISO8859_15;
 
-    fix[ 133 << 16] = EITFixUp::kEFixPro7Sat; // Premiere and Pro7/Sat.1
+    // Premiere and Pro7/Sat.1
+    fix[133 << 16] = EITFixUp::kEFixForceISO8859_15;
+
+    // DVB-T Spain
+    fix[       10LL << 32 | 8916 << 16 ] = // national multiplex
+        fix[   12LL << 32 | 8916 << 16 ] = // national multiplex
+        fix[   13LL << 32 | 8916 << 16 ] = // national multiplex
+        fix[ 6200LL << 32 | 8916 << 16 ] = // Madrid (TeleMadrid)
+        EITFixUp::kEFixForceISO8859_15;
 }
 
 static int calc_eit_utc_offset(void)
 {
     QString config_offset = gContext->GetSetting("EITTimeOffset", "Auto");
+
     if (config_offset == "Auto")
-    {
-        QDateTime loc = QDateTime::currentDateTime(Qt::LocalTime);
-        QDateTime utc = QDateTime::currentDateTime(Qt::UTC);
+        return calc_utc_offset();
 
-        int utc_offset = MythSecsTo(utc, loc);
-
-        // clamp to nearest minute if within 10 seconds
-        int off = utc_offset % 60;
-        if (abs(off) < 10)
-            utc_offset -= off;
-        if (off < -50 && off > -60)
-            utc_offset -= 60 + off;
-        if (off > +50 && off < +60)
-            utc_offset += 60 - off;
-
-        return utc_offset;
-    }
     if (config_offset == "None")
         return 0;
 
