@@ -7,17 +7,18 @@
 #include "scheduledrecording.h"
 #include "recordingprofile.h"
 #include "playgroup.h"
+#include "storagegroup.h"
 
-class SimpleSRSetting: public SimpleDBStorage
+class SimpleSRStorage : public SimpleDBStorage
 {
-    protected:
-        SimpleSRSetting(ScheduledRecording *_parent, QString name)
-                      : SimpleDBStorage("record", name),
-                        parent(_parent)
-        {
-            parent->addChild(this);
-            setName(name);
-        }
+  protected:
+    SimpleSRStorage(Setting *_setting, ScheduledRecording *_parent,
+                    QString name) :
+        SimpleDBStorage(_setting, "record", name), parent(_parent)
+    {
+        parent->addChild(_setting);
+        _setting->setName(name);
+    }
 
         virtual QString setClause(MSqlBindings& bindings)
         {
@@ -28,7 +29,7 @@ class SimpleSRSetting: public SimpleDBStorage
                     getColumn() + " = " + colTag);
 
             bindings.insert(recordidTag, parent->getRecordID());
-            bindings.insert(colTag, getValue().utf8());
+            bindings.insert(colTag, setting->getValue().utf8());
 
             return query;
         }
@@ -303,11 +304,16 @@ class SRDupIn : public SRSelectSetting
         SRDupIn(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "dupInList", "[ Check for duplicates in ]", _group, "dupin", _list )
         {
-            if (gContext->GetNumSetting("HaveRepeats", 0))
-                addSelection(QObject::tr("Record new episodes only"), kDupsNewEpi);
             addSelection(QObject::tr("Look for duplicates in current and previous recordings"), kDupsInAll);
             addSelection(QObject::tr("Look for duplicates in current recordings only"), kDupsInRecorded);
             addSelection(QObject::tr("Look for duplicates in previous recordings only"), kDupsInOldRecorded);
+            addSelection(QObject::tr("Exclude unidentified episodes"), kDupsExGeneric | kDupsInAll);
+            if (gContext->GetNumSetting("HaveRepeats", 0))
+            {
+                addSelection(QObject::tr("Exclude old episodes"), kDupsExRepeats | kDupsInAll);
+                addSelection(QObject::tr("Record new episodes only"), kDupsNewEpi | kDupsInAll);
+                addSelection(QObject::tr("Record new episode first showings"), kDupsFirstNew | kDupsInAll);
+            }
             setValue(kDupsInAll);
             _parent->setDupInObj(this);
         }
@@ -322,6 +328,7 @@ class SRDupMethod: public SRSelectSetting
                               "dupmethod", _list)
         {
             addSelection(QObject::tr("Match duplicates using subtitle & description"), kDupCheckSubDesc);
+            addSelection(QObject::tr("Match duplicates using subtitle then description"), kDupCheckSubThenDesc);
             addSelection(QObject::tr("Match duplicates using subtitle"), kDupCheckSub);
             addSelection(QObject::tr("Match duplicates using description"), kDupCheckDesc);
             addSelection(QObject::tr("Don't match duplicates"), kDupCheckNone);
@@ -331,17 +338,19 @@ class SRDupMethod: public SRSelectSetting
 };
 
 
-class SRRecSearchType: public IntegerSetting, public SimpleSRSetting
+class SRRecSearchType : public IntegerSetting, public SimpleSRStorage
 {
-    public:
-        SRRecSearchType(ScheduledRecording *parent):
-            SimpleSRSetting(parent, "search") {
-            setVisible(false);
-        }
+  public:
+    SRRecSearchType(ScheduledRecording *parent) :
+        IntegerSetting(this), SimpleSRStorage(this, parent, "search")
+    {
+        setVisible(false);
+    }
 };
 
 
-class SRProfileSelector: public SRSelectSetting {
+class SRProfileSelector : public SRSelectSetting
+{
     public:
         SRProfileSelector(ScheduledRecording *_parent, ManagedList* _list, ManagedListGroup* _group)
             : SRSelectSetting(_parent, "profileList", QObject::tr("[ Select recording Profile ]"), _group,
@@ -447,6 +456,7 @@ class SRStorageOptionsGroup : public ManagedListGroup
         friend class RootSRGroup;
         class SRProfileSelector* profile;
         class SRRecGroup* recGroup;
+        class SRStorageGroup* storageGroup;
         class SRPlayGroup* playGroup;
         class SRAutoExpire* autoExpire;
         class SRMaxEpisodes* maxEpisodes;
@@ -456,161 +466,133 @@ class SRStorageOptionsGroup : public ManagedListGroup
 };
 
 
-class SRChannel: public ChannelSetting, public SimpleSRSetting
+class SRChannel : public ChannelSetting, public SimpleSRStorage
 {
-    public:
-        SRChannel(ScheduledRecording *parent): SimpleSRSetting(parent, "chanid") {
-            setVisible(false);
-        }
+  public:
+    SRChannel(ScheduledRecording *parent) :
+        ChannelSetting(this),
+        SimpleSRStorage(this, parent, "chanid") { setVisible(false); }
 };
 
-class SRStation: public LineEditSetting, public SimpleSRSetting
+class SRStation : public LineEditSetting, public SimpleSRStorage
 {
-    public:
-        SRStation(ScheduledRecording *parent): SimpleSRSetting(parent, "station") {
-            setVisible(false);
-        }
+  public:
+    SRStation(ScheduledRecording *parent) :
+        LineEditSetting(this),
+        SimpleSRStorage(this, parent, "station") { setVisible(false); }
 };
 
-class SRTitle: public LineEditSetting, public SimpleSRSetting
+class SRTitle : public LineEditSetting, public SimpleSRStorage
 {
-    public:
-        SRTitle(ScheduledRecording *parent)
-              : SimpleSRSetting(parent, "title")
-        {
-            setVisible(false);
-        }
+  public:
+    SRTitle(ScheduledRecording *parent) :
+        LineEditSetting(this),
+        SimpleSRStorage(this, parent, "title") { setVisible(false); }
 };
 
-class SRSubtitle: public LineEditSetting, public SimpleSRSetting
+class SRSubtitle : public LineEditSetting, public SimpleSRStorage
 {
-    public:
-        SRSubtitle(ScheduledRecording *parent)
-                 : SimpleSRSetting(parent, "subtitle")
-        {
-            setVisible(false);
-        }
+  public:
+    SRSubtitle(ScheduledRecording *parent) :
+        LineEditSetting(this),
+        SimpleSRStorage(this, parent, "subtitle") { setVisible(false); }
 };
 
-class SRDescription: public LineEditSetting, public SimpleSRSetting
+class SRDescription : public LineEditSetting, public SimpleSRStorage
 {
-    public:
-        SRDescription(ScheduledRecording *parent)
-                    : SimpleSRSetting(parent, "description")
-        {
-            setVisible(false);
-        }
+  public:
+    SRDescription(ScheduledRecording *parent) :
+        LineEditSetting(this),
+        SimpleSRStorage(this, parent, "description") { setVisible(false); }
 };
 
-class SRStartTime: public TimeSetting, public SimpleSRSetting
+class SRStartTime : public TimeSetting, public SimpleSRStorage
 {
-    public:
-        SRStartTime(ScheduledRecording *parent)
-                  : SimpleSRSetting(parent, "starttime")
-        {
-            setVisible(false);
-        }
+  public:
+    SRStartTime(ScheduledRecording *parent) :
+        TimeSetting(this),
+        SimpleSRStorage(this, parent, "starttime") { setVisible(false); }
 };
 
-class SRStartDate: public DateSetting, public SimpleSRSetting
+class SRStartDate : public DateSetting, public SimpleSRStorage
 {
-    public:
-        SRStartDate(ScheduledRecording *parent)
-                  : SimpleSRSetting(parent, "startdate")
-        {
-            setVisible(false);
-        }
+  public:
+    SRStartDate(ScheduledRecording *parent) :
+        DateSetting(this),
+        SimpleSRStorage(this, parent, "startdate") { setVisible(false); }
 };
 
-class SREndTime: public TimeSetting, public SimpleSRSetting
+class SREndTime : public TimeSetting, public SimpleSRStorage
 {
-    public:
-        SREndTime(ScheduledRecording *parent)
-                : SimpleSRSetting(parent, "endtime")
-        {
-            setVisible(false);
-        }
+  public:
+    SREndTime(ScheduledRecording *parent) :
+        TimeSetting(this),
+        SimpleSRStorage(this, parent, "endtime") { setVisible(false); }
 };
 
-class SREndDate: public DateSetting, public SimpleSRSetting
+class SREndDate : public DateSetting, public SimpleSRStorage
 {
-    public:
-        SREndDate(ScheduledRecording *parent)
-                : SimpleSRSetting(parent, "enddate")
-        {
-            setVisible(false);
-        }
+  public:
+    SREndDate(ScheduledRecording *parent) :
+        DateSetting(this),
+        SimpleSRStorage(this, parent, "enddate") { setVisible(false); }
 };
 
 
-class SRCategory: public LineEditSetting, public SimpleSRSetting
+class SRCategory : public LineEditSetting, public SimpleSRStorage
 {
-    public:
-        SRCategory(ScheduledRecording *parent)
-                 : SimpleSRSetting(parent, "category")
-        {
-            setVisible(false);
-        }
+  public:
+    SRCategory(ScheduledRecording *parent) :
+        LineEditSetting(this),
+        SimpleSRStorage(this, parent, "category") { setVisible(false); }
 };
 
-class SRSeriesid: public LineEditSetting, public SimpleSRSetting
+class SRSeriesid : public LineEditSetting, public SimpleSRStorage
 {
-    public:
-        SRSeriesid(ScheduledRecording *parent)
-                 : SimpleSRSetting(parent, "seriesid")
-        {
-            setVisible(false);
-        }
+  public:
+    SRSeriesid(ScheduledRecording *parent) :
+        LineEditSetting(this),
+        SimpleSRStorage(this, parent, "seriesid") { setVisible(false); }
 };
 
-class SRProgramid: public LineEditSetting, public SimpleSRSetting
+class SRProgramid : public LineEditSetting, public SimpleSRStorage
 {
-    public:
-        SRProgramid(ScheduledRecording *parent)
-                  : SimpleSRSetting(parent, "programid")
-        {
-            setVisible(false);
-        };
+  public:
+    SRProgramid(ScheduledRecording *parent) :
+        LineEditSetting(this),
+        SimpleSRStorage(this, parent, "programid") { setVisible(false); }
 };
 
-class SRFindDay: public IntegerSetting, public SimpleSRSetting
+class SRFindDay : public IntegerSetting, public SimpleSRStorage
 {
-    public:
-        SRFindDay(ScheduledRecording *parent)
-            : SimpleSRSetting(parent, "findday")
-        {
-            setVisible(false);
-        }
+  public:
+    SRFindDay(ScheduledRecording *parent) :
+        IntegerSetting(this),
+        SimpleSRStorage(this, parent, "findday") { setVisible(false); }
 };
 
-class SRFindTime: public TimeSetting, public SimpleSRSetting
+class SRFindTime : public TimeSetting, public SimpleSRStorage
 {
-    public:
-        SRFindTime(ScheduledRecording *parent)
-            : SimpleSRSetting(parent, "findtime")
-        {
-            setVisible(false);
-        }
+  public:
+    SRFindTime(ScheduledRecording *parent) :
+        TimeSetting(this),
+        SimpleSRStorage(this, parent, "findtime") { setVisible(false); }
 };
 
-class SRFindId: public IntegerSetting, public SimpleSRSetting
+class SRFindId : public IntegerSetting, public SimpleSRStorage
 {
-    public:
-        SRFindId(ScheduledRecording *parent)
-            : SimpleSRSetting(parent, "findid")
-        {
-            setVisible(false);
-        }
+  public:
+    SRFindId(ScheduledRecording *parent) :
+        IntegerSetting(this),
+        SimpleSRStorage(this, parent, "findid") { setVisible(false); }
 };
 
-class SRParentId: public IntegerSetting, public SimpleSRSetting
+class SRParentId : public IntegerSetting, public SimpleSRStorage
 {
-    public:
-        SRParentId(ScheduledRecording *parent)
-            : SimpleSRSetting(parent, "parentid")
-        {
-            setVisible(false);
-        }
+  public:
+    SRParentId(ScheduledRecording *parent) :
+        IntegerSetting(this),
+        SimpleSRStorage(this, parent, "parentid") { setVisible(false); }
 };
 
 class SRAutoTranscode: public SRSelectSetting 
@@ -741,8 +723,8 @@ class SRInactive: public SRBoolSetting
 {
     public:
         SRInactive(ScheduledRecording *_parent, ManagedListGroup* _group, ManagedList* _list)
-                    : SRBoolSetting(_parent, QObject::tr("This recording schedule is inactive"), 
-                                     QObject::tr("This recording schedule is active"),
+                    : SRBoolSetting(_parent, QObject::tr("This recording rule is inactive"), 
+                                     QObject::tr("This recording rule is active"),
                                     "inactiveItem", "inactive", _group, _list )
         {
             _parent->setInactiveObj(this);
@@ -866,7 +848,7 @@ class SRRecGroup: public SRSelectSetting
                 if (!foundDefault && *it > QObject::tr("Default"))
                 {
                     addSelection(QObject::tr(
-                                 "Store in the \"%1\" recording group")
+                                 "Include in the \"%1\" recording group")
                                  .arg(QObject::tr("Default")), "Default");
                     foundDefault = true;
                 }
@@ -876,7 +858,7 @@ class SRRecGroup: public SRSelectSetting
                 else
                     dispValue = *it;
 
-                addSelection(QObject::tr("Store in the \"%1\" recording group")
+                addSelection(QObject::tr("Include in the \"%1\" recording group")
                                          .arg(dispValue), *it);
             }
 
@@ -888,6 +870,68 @@ class SRRecGroup: public SRSelectSetting
         void onGoingBack();
 };
 
+class SRStorageGroup: public SRSelectSetting
+{
+    Q_OBJECT
+
+    public:
+        SRStorageGroup(ScheduledRecording *_parent, ManagedList* _list,
+                       ManagedListGroup* _group)
+            : SRSelectSetting(_parent, "storagegroupList", QString("[ %1 ]")
+                              .arg(QObject::tr("Select Storage Group")),
+                              _group, "storagegroup", _list )
+        {
+            setValue("Default");
+            _parent->setStorageGroupObj(this);
+        }
+
+
+        virtual void load() {
+            fillSelections();
+            SRSelectSetting::load();
+        }
+
+        virtual QString getValue(void) const {
+            return settingValue;
+        }
+
+        virtual void fillSelections()
+        {
+            QStringList groups = StorageGroup::getRecordingsGroups();
+            QStringList::Iterator it;
+            QString value, dispValue;
+            bool foundDefault = false;
+
+            for (it = groups.begin(); it != groups.end(); ++it)
+            {
+                if (*it == "Default")
+                    foundDefault = true;
+            }
+
+            for (it = groups.begin(); it != groups.end(); ++it)
+            {
+                if (!foundDefault && *it > QObject::tr("Default"))
+                {
+                    addSelection(QObject::tr(
+                                 "Store in the \"%1\" storage group")
+                                 .arg(QObject::tr("Default")), "Default");
+                    foundDefault = true;
+                }
+
+                if (*it == "Default")
+                    dispValue = QObject::tr("Default");
+                else if (*it == "LiveTV")
+                    dispValue = QObject::tr("LiveTV");
+                else
+                    dispValue = *it;
+
+                addSelection(QObject::tr("Store in the \"%1\" storage group")
+                                         .arg(dispValue), *it);
+            }
+
+        }
+};
+
 class SRPlayGroup: public SRSelectSetting
 {
     Q_OBJECT
@@ -941,3 +985,5 @@ class SRInput: public SRSelectSetting
 };
 
 #endif
+
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
