@@ -7,11 +7,12 @@
 #include "DisplayRes.h"
 #include "videooutbase.h"
 
+#include "util-x11.h"
 #include <X11/Xatom.h>
 #include <X11/Xutil.h>
+#include <X11/extensions/Xvlib.h>
 #include <X11/extensions/XShm.h>
 #include <X11/extensions/Xv.h>
-#include <X11/extensions/Xvlib.h>
 
 #undef HAVE_AV_CONFIG_H
 extern "C" {
@@ -21,6 +22,9 @@ extern "C" {
 class NuppelVideoPlayer;
 class ChromaKeyOSD;
 
+class OpenGLContext;
+class OpenGLVideo;
+
 class XvMCBufferSettings;
 class XvMCSurfaceTypes;
 class XvMCTextures;
@@ -43,7 +47,7 @@ class XvMCOSD;
 
 typedef enum VideoOutputSubType
 {
-    XVUnknown = 0, Xlib, XShm, XVideo, XVideoMC, XVideoIDCT, XVideoVLD,
+    XVUnknown = 0, Xlib, XShm, OpenGL, XVideo, XVideoMC, XVideoIDCT, XVideoVLD,
 } VOSType;
 
 class VideoOutputXv : public VideoOutput
@@ -72,16 +76,22 @@ class VideoOutputXv : public VideoOutput
     void ClearAfterSeek(void);
 
     void MoveResize(void);
-    void InputChanged(int width, int height, float aspect, 
-                      MythCodecID av_codec_id);
-    void Zoom(int direction);
+    bool InputChanged(const QSize &input_size,
+                      float        aspect,
+                      MythCodecID  av_codec_id,
+                      void        *codec_private);
+    void Zoom(ZoomDirection direction);
     void VideoAspectRatioChanged(float aspect);
     void EmbedInWidget(WId wid, int x, int y, int w, int h);
     void StopEmbedding(void);
+    void ResizeForGui(void); 
+    void ResizeForVideo(void);
     void DrawUnusedRects(bool sync = true);
     void UpdatePauseFrame(void);
-    int  SetPictureAttribute(int attribute, int newValue);
+    int  SetPictureAttribute(PictureAttribute attribute, int newValue);
+    void InitPictureAttributes(void);
 
+    /// Monitor refresh time in microseconds
     int  GetRefreshRate(void);
 
     virtual bool hasMCAcceleration(void) const
@@ -92,11 +102,14 @@ class VideoOutputXv : public VideoOutput
         { return XVideoVLD == VideoOutputSubType(); }
 
     void CheckFrameStates(void);
+    QRect GetPIPRect(int location, NuppelVideoPlayer *pipplayer = NULL);
+
+    virtual void ShutdownVideoResize(void);
 
     static MythCodecID GetBestSupportedCodec(uint width, uint height,
                                              uint osd_width, uint osd_height,
                                              uint stream_type, int xvmc_chroma,
-                                             bool test_surface);
+                                             bool test_surface, bool force_xv);
 
     static int GrabSuitableXvPort(Display* disp, Window root,
                                   MythCodecID type,
@@ -110,9 +123,13 @@ class VideoOutputXv : public VideoOutput
                                           int width, int height);
     static void DeleteXvMCContext(Display* disp, XvMCContext*& ctx);
 
+
+    static QStringList GetAllowedRenderers(MythCodecID myth_codec_id,
+                                           const QSize &video_dim);
+
   private:
     VOSType VideoOutputSubType() const { return video_output_subtype; }
-    virtual QRect GetVisibleOSDBounds(float&, float&) const;
+    virtual QRect GetVisibleOSDBounds(float&, float&, float) const;
     virtual QRect GetTotalOSDBounds(void) const;
 
     VideoFrame *GetNextFreeFrame(bool allow_unsafe);
@@ -121,32 +138,45 @@ class VideoOutputXv : public VideoOutput
     void DoneDisplayingFrame(void);
 
     void ProcessFrameXvMC(VideoFrame *frame, OSD *osd);
+    void ProcessFrameOpenGL(VideoFrame *frame, OSD *osd,
+                            FilterChain *filterList,
+                            NuppelVideoPlayer *pipPlayer);
     void ProcessFrameMem(VideoFrame *frame, OSD *osd,
                          FilterChain *filterList,
                          NuppelVideoPlayer *pipPlayer);
 
     void PrepareFrameXvMC(VideoFrame *, FrameScanType);
     void PrepareFrameXv(VideoFrame *);
+    void PrepareFrameOpenGL(VideoFrame *, FrameScanType);
     void PrepareFrameMem(VideoFrame *, FrameScanType);
 
     void ShowXvMC(FrameScanType scan);
     void ShowXVideo(FrameScanType scan);
 
+    void ShowPip(VideoFrame *frame, NuppelVideoPlayer *pipplayer);
+    virtual int DisplayOSD(VideoFrame *frame, OSD *osd,
+                           int stride = -1, int revision = -1);
+
     void ResizeForVideo(uint width, uint height);
     void InitDisplayMeasurements(uint width, uint height);
     void InitColorKey(bool turnoffautopaint);
 
-    bool InitVideoBuffers(MythCodecID, bool use_xv, bool use_shm);
+    bool InitVideoBuffers(MythCodecID, bool use_xv,
+                          bool use_shm, bool use_opengl);
+
     bool InitXvMC(MythCodecID);
     bool InitXVideo(void);
+    bool InitOpenGL(void);
     bool InitXShm(void);
     bool InitXlib(void);
+    bool InitOSD(const QString&);
+    bool CheckOSDInit(void);
 
     bool CreateXvMCBuffers(void);
     bool CreateBuffers(VOSType subtype);
-    vector<void*> CreateXvMCSurfaces(uint num, bool create_xvmc_blocks);
+    vector<void*> CreateXvMCSurfaces(uint num, bool surface_has_vld);
     vector<unsigned char*> CreateShmImages(uint num, bool use_xv);
-    void CreatePauseFrame(void);
+    void CreatePauseFrame(VOSType subtype);
     void CopyFrame(VideoFrame *to, const VideoFrame *from);
 
     void DeleteBuffers(VOSType subtype, bool delete_pause_frame);
@@ -164,6 +194,12 @@ class VideoOutputXv : public VideoOutput
     void ReturnAvailableOSD(XvMCOSD*);
 #endif // USING_XVMC
 
+    // OpenGL specific helper functions
+    bool SetDeinterlacingEnabledOpenGL(bool enable);
+    bool SetupDeinterlaceOpenGL(
+        bool interlaced, const QString &overridefilter);
+
+
     // Misc.
     MythCodecID          myth_codec_id;
     VOSType              video_output_subtype;
@@ -180,6 +216,7 @@ class VideoOutputXv : public VideoOutput
     int                  XJ_screen_num;
     unsigned long        XJ_white;
     unsigned long        XJ_black;
+    unsigned long        XJ_letterbox_colour;
     int                  XJ_depth;
     int                  XJ_screenx;
     int                  XJ_screeny;
@@ -222,12 +259,21 @@ class VideoOutputXv : public VideoOutput
     int                  xv_chroma;
     buffer_map_t         xv_buffers;
     bool                 xv_need_bobdeint_repaint;
+    QMap<PictureAttribute,int> xv_attribute_min;
+    QMap<PictureAttribute,int> xv_attribute_max;
+
+    // OpenGL drawing info
+    QMutex               gl_context_lock;
+    OpenGLContext       *gl_context;
+    OpenGLVideo         *gl_videochain;
+    OpenGLVideo         *gl_pipchain;
+    OpenGLVideo         *gl_osdchain;
+    bool                 gl_use_osd_opengl2;
+    bool                 gl_pip_ready;
+    bool                 gl_osd_ready;
 
     // Chromakey OSD info
     ChromaKeyOSD        *chroma_osd;
 };
 
-CodecID myth2av_codecid(MythCodecID codec_id,
-                        bool& vld, bool& idct, bool& mc);
-
 #endif // VIDEOOUT_XV_H_
