@@ -6,6 +6,8 @@
 #include <qpixmap.h>
 #include <qkeysequence.h>
 #include <qpaintdevicemetrics.h>
+#include <qdir.h>
+#include <qfile.h>
 #ifdef QWS
 #include <qwindowsystem_qws.h>
 #endif
@@ -15,6 +17,9 @@
 
 #include <pthread.h>
 
+#include <algorithm>
+using namespace std;
+
 #ifdef USE_LIRC
 #include "lirc.h"
 #include "lircevent.h"
@@ -51,11 +56,12 @@
 #ifdef USE_LIRC
 static void *SpawnLirc(void *param)
 {
-    MythMainWindow *main_window = (MythMainWindow *)param;
     QString config_file = MythContext::GetConfDir() + "/lircrc";
-    QString program("mythtv");
-    LircClient *cl = new LircClient(main_window);
-    if (!cl->Init(config_file, program))
+    if (!QFile::exists(config_file))
+        config_file = QDir::homeDirPath() + "/.lircrc";
+
+    LircClient *cl = new LircClient((MythMainWindow *)param);
+    if (!cl->Init(config_file, "mythtv"))
         cl->Process();
 
     return NULL;
@@ -118,6 +124,8 @@ struct JumpData
     void (*callback)(void);
     QString destination;
     QString description;
+    bool exittomain;
+    QString localAction;
 };
 
 struct MHData
@@ -153,6 +161,8 @@ class MythMainWindowPrivate
     bool exitingtomain;
     bool popwindows;
 
+    bool m_useDB;              ///< To allow or prevent database access
+
     QDict<KeyContext> keyContexts;
     QMap<int, JumpData*> jumpMap;
     QMap<QString, JumpData> destinationMap;
@@ -209,13 +219,25 @@ int MythMainWindowPrivate::TranslateKeyNum(QKeyEvent* e)
 }
 
 static MythMainWindow *mainWin = NULL;
+static QMutex mainLock;
 
-MythMainWindow *MythMainWindow::getMainWindow(void)
+/**
+ * \brief Return the existing main window, or create one
+ * \param useDB
+ *        If this is a temporary window, which is used to bootstrap
+ *        the database, passing false prevents any database access.
+ *
+ * \sa    MythContextPrivate::TempMainWindow()
+ */
+MythMainWindow *MythMainWindow::getMainWindow(const bool useDB)
 {
+    if (mainWin)
+        return mainWin;
+
+    QMutexLocker lock(&mainLock);
+
     if (!mainWin)
-    {
-        mainWin = new MythMainWindow();
-    }
+        mainWin = new MythMainWindow(useDB);
 
     return mainWin;
 }
@@ -246,7 +268,7 @@ MythPainter *GetMythPainter(void)
 #define QWidget QGLWidget
 #endif
 
-MythMainWindow::MythMainWindow()
+MythMainWindow::MythMainWindow(const bool useDB)
               : QWidget(NULL, "mainWindow")
 {
 
@@ -255,6 +277,9 @@ MythMainWindow::MythMainWindow()
 
     d->AllowInput = false;
 
+    // This prevents database errors from RegisterKey() when there is no DB:
+    d->m_useDB = useDB;
+
 
     QString painter = gContext->GetSetting("ThemePainter", "qt");
 #ifdef USE_OPENGL_PAINTER
@@ -270,7 +295,7 @@ MythMainWindow::MythMainWindow()
         d->painter = new MythQtPainter();
     }
 
-    Init();
+    //Init();
 
     d->ignore_lirc_keys = false;
     d->ignore_joystick_keys = false;
@@ -289,8 +314,8 @@ MythMainWindow::MythMainWindow()
     pthread_attr_t attr;
     pthread_attr_init(&attr);
     pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
-
     pthread_create(&lirc_tid, &attr, SpawnLirc, this);
+    pthread_attr_destroy(&attr);
 #endif
 
 #ifdef USE_JOYSTICK_MENU
@@ -300,8 +325,8 @@ MythMainWindow::MythMainWindow()
     pthread_attr_t attr2;
     pthread_attr_init(&attr2);
     pthread_attr_setdetachstate(&attr2, PTHREAD_CREATE_DETACHED);
-
     pthread_create(&js_tid, &attr2, SpawnJoystickMenu, this);
+    pthread_attr_destroy(&attr2);
 #endif
 
 #ifdef USING_APPLEREMOTE
@@ -310,8 +335,8 @@ MythMainWindow::MythMainWindow()
     pthread_attr_t attr3;
     pthread_attr_init(&attr3);
     pthread_attr_setdetachstate(&attr3, PTHREAD_CREATE_DETACHED);
-
     pthread_create(&appleremote_tid, &attr3, SpawnAppleRemote, this);
+    pthread_attr_destroy(&attr3);
 #endif
 
     d->keyContexts.setAutoDelete(true);
@@ -430,6 +455,12 @@ void MythMainWindow::drawTimeout(void)
 
     if (!d->painter->SupportsClipping())
         d->repaintRegion = d->repaintRegion.unite(d->uiScreenRect);
+    else
+    {
+        // Ensure that the region is not larger than the screen which
+        // can happen with bad themes
+        d->repaintRegion = d->repaintRegion.intersect(d->uiScreenRect);
+    }
 
     d->painter->Begin(this);
 
@@ -509,6 +540,8 @@ void MythMainWindow::Init(void)
     setGeometry(d->xbase, d->ybase, d->screenwidth, d->screenheight);
     setFixedSize(QSize(d->screenwidth, d->screenheight));
 
+    fonTweak = gContext->GetNumSetting("QtFonTweak", 0);
+
     bool hideCursor = gContext->GetNumSetting("HideMouseCursor", 1);
 #ifdef QWS
 #if QT_VERSION >= 0x030300
@@ -585,6 +618,18 @@ void MythMainWindow::Show(void)
 /* FIXME compatability only */
 void MythMainWindow::attach(QWidget *child)
 {
+#ifdef USING_MINGW
+#warning TODO FIXME MythMainWindow::attach() does not always work on MS Windows!
+    // if windows are created on different threads,
+    // or if setFocus() is called from a thread other than the main UI thread,
+    // setFocus() hangs the thread that called it
+    // currently, it's impossible to switch to program guide from livetv
+    VERBOSE(VB_IMPORTANT,
+            QString("MythMainWindow::attach old: %1, new: %2, thread: %3")
+            .arg(currentWidget() ? currentWidget()->name() : "none")
+            .arg(child->name())
+            .arg(::GetCurrentThreadId()));
+#endif
     if (currentWidget())
         currentWidget()->setEnabled(false);
 
@@ -595,13 +640,16 @@ void MythMainWindow::attach(QWidget *child)
 
 void MythMainWindow::detach(QWidget *child)
 {
-    if (d->widgetList.back() != child)
+    vector<QWidget*>::iterator it = 
+        std::find(d->widgetList.begin(), d->widgetList.end(), child);
+
+    if (it == d->widgetList.end())
     {
-        //cerr << "Not removing top-most widget\n";
+        VERBOSE(VB_IMPORTANT, "Could not find widget to detach");
         return;
     }
 
-    d->widgetList.pop_back();
+    d->widgetList.erase(it);
     QWidget *current = currentWidget();
 
     if (current)
@@ -708,6 +756,24 @@ bool MythMainWindow::TranslateKeyPress(const QString &context,
     actions.clear();
     int keynum = d->TranslateKeyNum(e);
 
+    QStringList localActions;
+    if (allowJumps && (d->jumpMap.count(keynum) > 0) && 
+        (d->jumpMap[keynum]->localAction != "") &&
+        (d->keyContexts[context]) &&
+        (d->keyContexts[context]->GetMapping(keynum, localActions)))
+    {
+        if (localActions.contains(d->jumpMap[keynum]->localAction))
+            allowJumps = false;
+    }
+
+    if (allowJumps && d->jumpMap.count(keynum) > 0 && 
+            !d->jumpMap[keynum]->exittomain && d->exitmenucallback == NULL)
+    {
+        void (*callback)(void) = d->jumpMap[keynum]->callback;
+        callback();
+        return false;
+    }
+
     if (allowJumps && 
         d->jumpMap.count(keynum) > 0 && d->exitmenucallback == NULL)
     {
@@ -799,19 +865,44 @@ void MythMainWindow::RegisterKey(const QString &context, const QString &action,
     QString keybind = key;
 
     MSqlQuery query(MSqlQuery::InitCon());
-    if (query.isConnected())
+
+    if (d->m_useDB && query.isConnected())
     {
-        query.prepare("SELECT keylist FROM keybindings WHERE "
+        query.prepare("SELECT keylist, description FROM keybindings WHERE "
                       "context = :CONTEXT AND action = :ACTION AND "
                       "hostname = :HOSTNAME ;");
         query.bindValue(":CONTEXT", context);
         query.bindValue(":ACTION", action);
         query.bindValue(":HOSTNAME", gContext->GetHostName());
 
-        if (query.exec() && query.isActive() && query.size() > 0)
+        bool ok = query.exec() && query.isActive();
+
+        if (ok && query.next())
         {
-            query.next();
             keybind = query.value(0).toString();
+            QString db_description = query.value(1).toString();
+
+            // Update keybinding description if changed
+            if (db_description != description)
+            {
+                VERBOSE(VB_IMPORTANT, "Updating description...");
+                query.prepare(
+                    "UPDATE keybindings "
+                    "SET description = :DESCRIPTION "
+                    "WHERE context   = :CONTEXT AND "
+                    "      action    = :ACTION  AND "
+                    "      hostname  = :HOSTNAME");
+
+                query.bindValue(":DESCRIPTION", description);
+                query.bindValue(":CONTEXT",     context);
+                query.bindValue(":ACTION",      action);
+                query.bindValue(":HOSTNAME",    gContext->GetHostName());
+
+                if (!query.exec())
+                {
+                    MythContext::DBError("Update Keybinding", query);
+                }
+            }
         }
         else
         {
@@ -916,7 +1007,8 @@ void MythMainWindow::BindJump(const QString &destination, const QString &key)
 
 void MythMainWindow::RegisterJump(const QString &destination, 
                                   const QString &description,
-                                  const QString &key, void (*callback)(void))
+                                  const QString &key, void (*callback)(void),
+                                  bool exittomain, QString localAction)
 {
     QString keybind = key;
 
@@ -941,7 +1033,7 @@ void MythMainWindow::RegisterJump(const QString &destination,
                           "keylist, hostname) VALUES ( :DEST, :DESC, :KEYLIST, "
                           ":HOST );");
             query.bindValue(":DEST", destination);
-            query.bindValue(":DEDC", description);
+            query.bindValue(":DESC", description);
             query.bindValue(":KEYLIST", inskey);
             query.bindValue(":HOST", gContext->GetHostName());
 
@@ -951,8 +1043,9 @@ void MythMainWindow::RegisterJump(const QString &destination,
             }
         }
     }
- 
-    JumpData jd = { callback, destination, description };
+
+    JumpData jd =
+        { callback, destination, description, exittomain, localAction };
     d->destinationMap[destination] = jd;
 
     BindJump(destination, keybind); 
@@ -982,12 +1075,16 @@ void MythMainWindow::RegisterMediaHandler(const QString &destination,
                                           int            mediaType,
                                           const QString &extensions)
 {
+#ifndef _WIN32
     if (d->mediaHandlerMap.count(destination) == 0) 
     {
         MHData mhd = { callback, mediaType, destination, description };
+        QString msg = MythMediaDevice::MediaTypeString((MediaType)mediaType);
+        if (extensions)
+            msg += QString(", ext(%1)").arg(extensions);
 
-        VERBOSE(VB_GENERAL, QString("Registering %1 as a media handler %2")
-                .arg(destination).arg(QString("ext(%1)").arg(extensions)));
+        VERBOSE(VB_MEDIA, "Registering " + destination +
+                          " as a media handler for " + msg);
 
         d->mediaHandlerMap[destination] = mhd;
 
@@ -1000,6 +1097,7 @@ void MythMainWindow::RegisterMediaHandler(const QString &destination,
        VERBOSE(VB_GENERAL, QString("%1 is already registered as a media "
                                    "handler.").arg(destination));
     }
+#endif // !_WIN32
 }
 
 void MythMainWindow::RegisterMediaPlugin(const QString &name, 
@@ -1072,9 +1170,10 @@ bool MythMainWindow::eventFilter(QObject *, QEvent *e)
     {
         case QEvent::KeyPress:
         {
+            QKeyEvent *ke = dynamic_cast<QKeyEvent*>(e);
+
             if (currentWidget())
             {
-                QKeyEvent *ke = dynamic_cast<QKeyEvent*>(e);
                 ke->accept();
                 QWidget *current = currentWidget();
                 if (current && current->isEnabled())
@@ -1091,7 +1190,7 @@ bool MythMainWindow::eventFilter(QObject *, QEvent *e)
                 MythScreenType *top = (*it)->GetTopScreen();
                 if (top)
                 {
-                    if (top->keyPressEvent(dynamic_cast<QKeyEvent*>(e)))
+                    if (top->keyPressEvent(ke))
                         return true;
                 }
             }
@@ -1164,6 +1263,34 @@ bool MythMainWindow::eventFilter(QObject *, QEvent *e)
             }
             break;
         }
+        case QEvent::Wheel:
+        {
+            QWheelEvent* qmw = dynamic_cast<QWheelEvent*>(e);
+            int delta = qmw->delta();
+            if (delta>0)
+            {
+                qmw->accept();
+                QKeyEvent *key = new QKeyEvent(QEvent::KeyPress, Qt::Key_Up, 0,
+                                               Qt::NoButton);
+                QObject *key_target = getTarget(*key);
+                if (!key_target)
+                    QApplication::postEvent(this, key);
+                else
+                    QApplication::postEvent(key_target, key);
+            }
+            if (delta<0)
+            {
+                qmw->accept();
+                QKeyEvent *key = new QKeyEvent(QEvent::KeyPress, Qt::Key_Down,
+                                               0, Qt::NoButton);
+                QObject *key_target = getTarget(*key);
+                if (!key_target)
+                    QApplication::postEvent(this, key);
+                else
+                    QApplication::postEvent(key_target, key);
+            }
+            break;
+        }
         default:
             break;
     }
@@ -1205,6 +1332,7 @@ void MythMainWindow::customEvent(QCustomEvent *ce)
         else
             QApplication::sendEvent(key_target, &key);
     }
+#ifndef _WIN32
     else if (ce->type() == kMediaEventType) 
     {
         MediaEvent *media_event = (MediaEvent*)ce;
@@ -1217,6 +1345,23 @@ void MythMainWindow::customEvent(QCustomEvent *ce)
 
         if (pDev)
         {
+            if (!pDev->isUsable())
+            {
+                // We don't want to jump to the main window, but should
+                // call each plugin's callback so it can track this change.
+                // Should also do MediaMonitor::ValidateAndLock(pDev) first
+
+                while (itr != d->mediaHandlerMap.end())
+                {
+                    if (itr.data().MediaType & (int)pDev->getMediaType())
+                        itr.data().callback(pDev);
+
+                    itr++;
+                }
+
+                return;
+            }
+
             /* FIXME, this needs rewritten */
             QWidget * activewidget = qApp->focusWidget();
             MythDialog * activedialog = NULL;
@@ -1251,6 +1396,8 @@ void MythMainWindow::customEvent(QCustomEvent *ce)
                 mon->Unlock(pDev);
         }
     }
+#endif // !_WIN32
+
 #if defined(USE_LIRC) || defined(USING_APPLEREMOTE)
     else if (ce->type() == kLircKeycodeEventType && !d->ignore_lirc_keys) 
     {
@@ -1410,15 +1557,36 @@ int MythMainWindow::NormalizeFontSize(int pointSize)
 {
     QPaintDeviceMetrics pdm(this);
 
+    float floatSize = pointSize;
     float desired = 100.0;
-    pointSize = (int)(pointSize * desired / pdm.logicalDpiY());
-    pointSize = (int)(pointSize * d->hmult);
+
+#ifdef USING_MINGW
+    // logicalDpiY not supported in QT3/win.
+    int logicalDpiY = 100;
+    HDC hdc = GetDC(NULL);
+    if (hdc)
+    {
+        logicalDpiY = GetDeviceCaps(hdc, LOGPIXELSY);
+        ReleaseDC(NULL, hdc);
+    }
+#else
+    int logicalDpiY = pdm.logicalDpiY();
+#endif
+
+    // adjust for screen resolution relative to 100 dpi
+    floatSize = floatSize * desired / logicalDpiY;
+    // adjust for myth GUI size relative to 800x600
+    floatSize = floatSize * d->hmult;
+    // adjust by the configurable fine tuning percentage
+    floatSize = floatSize * ((100.0 + fonTweak) / 100.0);
+    // round to the nearest point size
+    pointSize = (int)(floatSize + 0.5);
 
     return pointSize;
 }
 
-QFont MythMainWindow::CreateFont(const QString &face, int pointSize, 
-                                 int weight, bool italic)
+QFont MythMainWindow::CreateQFont(const QString &face, int pointSize, 
+                                  int weight, bool italic)
 {
     QFont font = QFont(face);
     if (!font.exactMatch())
@@ -1475,3 +1643,4 @@ QRect MythMainWindow::GetUIScreenRect(void)
     return d->uiScreenRect;
 }
 
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
