@@ -15,10 +15,17 @@ using namespace std;
 #include <OpenGL/glext.h>
 #endif
 
+#ifdef _WIN32
+#include <GL/glext.h>
+#endif
+
 #include "mythcontext.h"
 #include "mythpainter_ogl.h"
 #include "mythfontproperties.h"
 
+#define MAX_GL_ITEMS 128
+#define MAX_STRING_ITEMS 48
+
 #ifndef GL_TEXTURE_RECTANGLE_ARB
 #define GL_TEXTURE_RECTANGLE_ARB 0x84F5 
 #endif
@@ -31,37 +38,13 @@ using namespace std;
 #define GL_TEXTURE_RECTANGLE_NV 0x84F5
 #endif
 
-class StringImageCache : public QCache<MythImage>
-{
-  public:
-    StringImageCache() : QCache<MythImage>(25, 31) 
-    {  setAutoDelete(true); }
-};
-
-class IntImageCache : public QIntCache<MythImage>
-{
-  public:
-    IntImageCache(MythOpenGLPainter *parent) 
-          : QIntCache<MythImage>(25, 31) 
-    { m_parent = parent; setAutoDelete(false); }
-
-  protected:
-    void deleteItem(MythImage *im) { printf("deleting item\n"); m_parent->DeleteFormatImage(im); }
-
-    MythOpenGLPainter *m_parent;   
-};
-
 MythOpenGLPainter::MythOpenGLPainter() 
                  : MythPainter()
 {
-    m_StringImageCache = new StringImageCache();
-    m_IntImageCache = new IntImageCache(this);
 }
 
 MythOpenGLPainter::~MythOpenGLPainter()
 {
-    delete m_StringImageCache;
-    delete m_IntImageCache;
 }
 
 void MythOpenGLPainter::Begin(QWidget *parent)
@@ -80,10 +63,16 @@ void MythOpenGLPainter::Begin(QWidget *parent)
     glViewport(0, 0, parent->width(), parent->height());
     glMatrixMode(GL_PROJECTION);
     glLoadIdentity();
-    glOrtho(0, 800, 600, 0, -999999, 999999);
+    glOrtho(0, parent->width(), parent->height(), 0, -999999, 999999);
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
-    glTranslatef(0.375, 0.375, 0.0);
+    //glTranslatef(0.2, 0.2, 0.0);
+
+    GLint param;
+    glGetIntegerv(GL_MAX_TEXTURE_SIZE, &param);
+    m_maxTexDim = param;
+    if (m_maxTexDim == 0)
+        m_maxTexDim = 512;
 }
 
 void MythOpenGLPainter::End(void)
@@ -103,8 +92,9 @@ void MythOpenGLPainter::End(void)
 int MythOpenGLPainter::NearestGLTextureSize(int v)
 {
     int n = 0, last = 0;
+    int s;
 
-    for (int s = 0; s < 32; ++s) 
+    for (s = 0; s < 32; ++s) 
     {
         if (((v >> s) & 1) == 1) 
         {
@@ -114,9 +104,11 @@ int MythOpenGLPainter::NearestGLTextureSize(int v)
     }
 
     if (n > 1)
-        return 1 << (last + 1);
+        s = 1 << (last + 1);
+    else
+        s = 1 << last;
 
-    return 1 << last;
+    return min(s, m_maxTexDim);
 }
 
 void MythOpenGLPainter::RemoveImageFromCache(MythImage *im)
@@ -128,6 +120,8 @@ void MythOpenGLPainter::RemoveImageFromCache(MythImage *im)
 
         glDeleteTextures(1, textures);
         m_ImageIntMap.erase(im);
+
+        m_ImageExpireList.remove(im);
     }
 }
 
@@ -177,7 +171,8 @@ void MythOpenGLPainter::BindTextureFromCache(MythImage *im,
 
         if (!im->IsChanged())
         {
-            m_IntImageCache->find(val);
+            m_ImageExpireList.remove(im);
+            m_ImageExpireList.push_back(im);
             glBindTexture(q_gl_texture, val);
             return;
         }
@@ -229,7 +224,14 @@ void MythOpenGLPainter::BindTextureFromCache(MythImage *im,
                  GL_RGBA, GL_UNSIGNED_BYTE, tx.bits());
 
     m_ImageIntMap[im] = tx_id;
-    m_IntImageCache->insert((long)tx_id, im);
+    m_ImageExpireList.push_back(im);
+
+    if (m_ImageExpireList.size() > MAX_GL_ITEMS)
+    {
+        MythImage *im = m_ImageExpireList.front();
+        m_ImageExpireList.pop_front();
+        RemoveImageFromCache(im);
+    }
 }
 
 void MythOpenGLPainter::DrawImage(const QRect &r, MythImage *im, 
@@ -284,17 +286,23 @@ int MythOpenGLPainter::CalcAlpha(int alpha1, int alpha2)
     return (int)(alpha1 * (alpha2 / 255.0));
 }
 
-// XXX: Need to hash string with font properties
 MythImage *MythOpenGLPainter::GetImageFromString(const QString &msg, 
                                                  int flags, const QRect &r, 
                                                  const MythFontProperties &font)
 {
-    MythImage *im;
+    QString incoming = font.GetHash() + QString::number(r.width()) + 
+                       QString::number(r.height()) + QString::number(flags) +
+                       msg;
 
-    if ((im = m_StringImageCache->find(msg)))
-        return im;
+    if (m_StringToImageMap.contains(incoming))
+    {
+        m_StringExpireList.remove(incoming);
+        m_StringExpireList.push_back(incoming);
 
-    im = GetFormatImage();
+        return m_StringToImageMap[incoming];
+    }
+
+    MythImage *im = GetFormatImage();
 
     qApp->lock();
 
@@ -315,7 +323,7 @@ MythImage *MythOpenGLPainter::GetImageFromString(const QString &msg,
     pm.fill();
 
     QPainter tmp(&pm);
-    tmp.setFont(font.face);
+    tmp.setFont(font.face());
     tmp.setPen(Qt::black);
     tmp.drawText(0, 0, r.width(), r.height(), flags, msg);
     tmp.end();
@@ -335,7 +343,23 @@ MythImage *MythOpenGLPainter::GetImageFromString(const QString &msg,
         colorTable[i] = qRgba(0, 0, 0, alpha);
     }
 
-    m_StringImageCache->insert(msg, im);
+    m_StringToImageMap[incoming] = im;
+    m_StringExpireList.push_back(incoming);
+
+    if (m_StringExpireList.size() > MAX_STRING_ITEMS)
+    {
+        QString oldmsg = m_StringExpireList.front();
+        m_StringExpireList.pop_front();
+
+        MythImage *oldim = NULL;
+        if (m_StringToImageMap.contains(oldmsg))
+            oldim = m_StringToImageMap[oldmsg];
+
+        m_StringToImageMap.erase(oldmsg);
+
+        if (oldim)
+            oldim->DownRef();
+    }
 
     return im;
 }
@@ -399,50 +423,62 @@ void MythOpenGLPainter::DrawText(const QRect &r, const QString &msg,
     newRect.setWidth(im->width());
     newRect.setHeight(im->height());
 
-    if (font.hasShadow)
+    if (font.hasShadow())
     {
-        QRect a = newRect;
-        a.moveBy(font.shadowOffset.x(), font.shadowOffset.y());
+        QPoint shadowOffset;
+        QColor shadowColor;
+        int shadowAlpha;
 
-        ReallyDrawText(font.shadowColor, a, CalcAlpha(font.shadowAlpha, alpha));
+        font.GetShadow(shadowOffset, shadowColor, shadowAlpha);
+
+        QRect a = newRect;
+        a.moveBy(shadowOffset.x(), shadowOffset.y());
+
+        ReallyDrawText(shadowColor, a, CalcAlpha(shadowAlpha, alpha));
     }
 
-    if (font.hasOutline)
+    if (font.hasOutline())
     {
+        QColor outlineColor;
+        int outlineSize, outlineAlpha;
+
+        font.GetOutline(outlineColor, outlineSize, outlineAlpha);
+
+        /* FIXME: use outlineAlpha */
         int outalpha = alpha;
         //if (alpha < 255)
             outalpha /= 16;
 
         QRect a = newRect;
-        a.moveBy(0 - font.outlineSize, 0 - font.outlineSize);
-        ReallyDrawText(font.outlineColor, a, outalpha);
+        a.moveBy(0 - outlineSize, 0 - outlineSize);
+        ReallyDrawText(outlineColor, a, outalpha);
 
-        for (int i = (0 - font.outlineSize + 1); i <= font.outlineSize; i++)
+        for (int i = (0 - outlineSize + 1); i <= outlineSize; i++)
         {
             a.moveBy(1, 0);
-            ReallyDrawText(font.outlineColor, a, outalpha);
+            ReallyDrawText(outlineColor, a, outalpha);
         }
 
-        for (int i = (0 - font.outlineSize + 1); i <= font.outlineSize; i++)
+        for (int i = (0 - outlineSize + 1); i <= outlineSize; i++)
         {
             a.moveBy(0, 1);
-            ReallyDrawText(font.outlineColor, a, outalpha);
+            ReallyDrawText(outlineColor, a, outalpha);
         }
 
-        for (int i = (0 - font.outlineSize + 1); i <= font.outlineSize; i++)
+        for (int i = (0 - outlineSize + 1); i <= outlineSize; i++)
         {
             a.moveBy(-1, 0);
-            ReallyDrawText(font.outlineColor, a, outalpha);
+            ReallyDrawText(outlineColor, a, outalpha);
         }
 
-        for (int i = (0 - font.outlineSize + 1); i <= font.outlineSize; i++)
+        for (int i = (0 - outlineSize + 1); i <= outlineSize; i++)
         {
             a.moveBy(0, -1);
-            ReallyDrawText(font.outlineColor, a, outalpha);
+            ReallyDrawText(outlineColor, a, outalpha);
         }
     }
 
-    ReallyDrawText(font.color, newRect, alpha);
+    ReallyDrawText(font.color(), newRect, alpha);
 }
 
 MythImage *MythOpenGLPainter::GetFormatImage()
