@@ -19,6 +19,7 @@
 #include <qregexp.h>
 
 #include <unistd.h>
+#include <stdlib.h>
 #include <qstringlist.h>
 #include <sys/time.h>
 
@@ -51,8 +52,8 @@ SSDP::SSDP( int nServicePort ) : m_bTermRequested( false )
     m_Sockets[ SocketIdx_Broadcast ]->setBlocking( FALSE );
 
     // Setup SearchSocket
-
-    m_Sockets[ SocketIdx_Search ]->bind( INADDR_ANY, m_nSearchPort ); 
+    QHostAddress ip4addr( (Q_UINT32) INADDR_ANY );
+    m_Sockets[ SocketIdx_Search ]->bind( ip4addr, m_nSearchPort ); 
 
     m_pNotifyTask   = NULL;
 
@@ -66,6 +67,9 @@ SSDP::~SSDP()
 {
     DisableNotifications();
 
+    m_bTermRequested = true;
+    wait();
+
     if (m_pNotifyTask != NULL)
         m_pNotifyTask->Release();
 
@@ -82,38 +86,11 @@ SSDP::~SSDP()
 //
 /////////////////////////////////////////////////////////////////////////////
 
-bool SSDP::IsTermRequested()
-{
-    m_lock.lock();
-    bool bTermRequested = m_bTermRequested;
-    m_lock.unlock();
-
-    return( bTermRequested );
-}
-
-/////////////////////////////////////////////////////////////////////////////
-//
-/////////////////////////////////////////////////////////////////////////////
-
-void SSDP::RequestTerminate(void)
-{
-    m_lock.lock();  
-    m_bTermRequested = true; 
-    m_lock.unlock();
-
-    // Call wait to give thread time to terminate.
-
-    wait( 500 );
-}
-
-/////////////////////////////////////////////////////////////////////////////
-//
-/////////////////////////////////////////////////////////////////////////////
-
 void SSDP::EnableNotifications()
 {
     if ( m_pNotifyTask == NULL )
     {
+        VERBOSE(VB_UPNP, "SSDP::EnableNotifications() - creating new task");
         m_pNotifyTask = new UPnpNotifyTask( m_nServicePort ); 
 
         // ------------------------------------------------------------------
@@ -126,6 +103,7 @@ void SSDP::EnableNotifications()
         // First Send out Notification that we are leaving the network.
         // ------------------------------------------------------------------
 
+        VERBOSE(VB_UPNP, "SSDP::EnableNotifications() - sending NTS_byebye");
         m_pNotifyTask->SetNTS( NTS_byebye );
         m_pNotifyTask->Execute( NULL );
     }
@@ -134,8 +112,10 @@ void SSDP::EnableNotifications()
     // Add Announcement Task to the Queue
     // ------------------------------------------------------------------
 
+    VERBOSE(VB_UPNP, "SSDP::EnableNotifications() - sending NTS_alive");
     m_pNotifyTask->SetNTS( NTS_alive );
     UPnp::g_pTaskQueue->AddTask( m_pNotifyTask  );
+    VERBOSE(VB_UPNP, "SSDP::EnableNotifications() - Task added to UPnP queue");
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -197,7 +177,7 @@ void SSDP::run()
     // Listen for new Requests
     // ----------------------------------------------------------------------
 
-    while (!IsTermRequested())
+    while ( ! m_bTermRequested )
     {
         int nMaxSocket = 0;
 
