@@ -53,7 +53,7 @@ QString UPnpCDSExtensionResults::GetResultXML()
 //
 /////////////////////////////////////////////////////////////////////////////
 
-UPnpCDS::UPnpCDS( UPnpDevice *pDevice ) : Eventing( "UPnpCDS", "CDS_Event" )
+UPnpCDS::UPnpCDS( UPnpDevice *pDevice, const QString &sSharePath ) : Eventing( "UPnpCDS", "CDS_Event" )
 {
     m_extensions.setAutoDelete( true );
 
@@ -71,8 +71,9 @@ UPnpCDS::UPnpCDS( UPnpDevice *pDevice ) : Eventing( "UPnpCDS", "CDS_Event" )
 
     SetValue< unsigned short >( "SystemUpdateID", 1 );
 
-    QString sUPnpDescPath = UPnp::g_pConfig->GetValue( "UPnP/DescXmlPath", m_sSharePath );
+    QString sUPnpDescPath = UPnp::g_pConfig->GetValue( "UPnP/DescXmlPath", sSharePath );
 
+    m_sSharePath           = sSharePath;
     m_sServiceDescFileName = sUPnpDescPath + "CDS_scpd.xml";
     m_sControlUrl          = "/CDS_Control";
 
@@ -185,6 +186,7 @@ void UPnpCDS::HandleBrowse( HTTPRequest *pRequest )
     UPnpCDSRequest           request;
 
     request.m_sObjectId         = pRequest->m_mapParams[ "ObjectID"      ];
+    request.m_sContainerID      = pRequest->m_mapParams[ "ContainerID"   ];
     request.m_sParentId         = "0";
     request.m_eBrowseFlag       = GetBrowseFlag( pRequest->m_mapParams[ "BrowseFlag"    ] );
     request.m_sFilter           = pRequest->m_mapParams[ "Filter"        ];
@@ -219,6 +221,10 @@ void UPnpCDS::HandleBrowse( HTTPRequest *pRequest )
     short          nUpdateID       = 0;
     QString        sResultXML;
 
+    VERBOSE(VB_UPNP, QString("UPnpCDS::HandleBrowse ObjectID=%1, ContainerId=%2")
+		             .arg(request.m_sObjectId)
+		             .arg(request.m_sContainerID));
+
     if (request.m_sObjectId == "0")
     {
         // ------------------------------------------------------------------
@@ -232,7 +238,7 @@ void UPnpCDS::HandleBrowse( HTTPRequest *pRequest )
                 // ----------------------------------------------------------------------
                 // Return Root Object Only
                 // ----------------------------------------------------------------------
-        
+
                 eErrorCode      = UPnPResult_Success;
                 nNumberReturned = 1;
                 nTotalMatches   = 1;
@@ -306,10 +312,8 @@ void UPnpCDS::HandleBrowse( HTTPRequest *pRequest )
 
         while (( pExtension != NULL ) && (pResult == NULL))
         {
-            if ( request.m_sObjectId.startsWith( pExtension->m_sExtensionId, true ))
-            {
-                pResult = pExtension->Browse(  &request );
-            }
+            VERBOSE(VB_UPNP, QString("UPNP Browse : Searching for : %1  / ObjectID : %2").arg(pExtension->m_sExtensionId).arg(request.m_sObjectId));
+            pResult = pExtension->Browse( &request );
 
             pExtension = m_extensions.next();
         }
@@ -371,13 +375,18 @@ void UPnpCDS::HandleSearch( HTTPRequest *pRequest )
     short         nUpdateID       = 0;
     QString       sResultXML;
 
-    request.m_sObjectId         = pRequest->m_mapParams[ "ContainerID"   ];
+    request.m_sObjectId         = pRequest->m_mapParams[ "ObjectID"      ];
+    request.m_sContainerID      = pRequest->m_mapParams[ "ContainerID"   ];
     request.m_sFilter           = pRequest->m_mapParams[ "Filter"        ];
     request.m_nStartingIndex    = pRequest->m_mapParams[ "StartingIndex" ].toLong();
     request.m_nRequestedCount   = pRequest->m_mapParams[ "RequestedCount"].toLong();
     request.m_sSortCriteria     = pRequest->m_mapParams[ "SortCriteria"  ];
     request.m_sSearchCriteria   = pRequest->m_mapParams[ "SearchCriteria"];
 
+    VERBOSE(VB_UPNP, QString("UPnpCDS::HandleSearch ObjectID=%1, ContainerId=%2")
+                        .arg(request.m_sObjectId)
+                        .arg(request.m_sContainerID));
+
     // ----------------------------------------------------------------------
     // Break the SearchCriteria into it's parts 
     // -=>TODO: This DOES NOT handle ('s or other complex expressions
@@ -414,7 +423,7 @@ void UPnpCDS::HandleSearch( HTTPRequest *pRequest )
 
     // ----------------------------------------------------------------------
 
-/*
+
     VERBOSE(VB_UPNP,QString("UPnpCDS::ProcessRequest \n"
                             ": url            = %1 \n"
                             ": Method         = %2 \n"
@@ -434,36 +443,35 @@ void UPnpCDS::HandleSearch( HTTPRequest *pRequest )
                        .arg( request.m_nRequestedCount)
                        .arg( request.m_sSortCriteria  )
                        .arg( request.m_sSearchClass   ));
-*/
+
 
     UPnpCDSExtension    *pExtension = m_extensions.first();
 
-    bool bSearchDone = false;
+    //bool bSearchDone = false;
 
-    while (( pExtension != NULL ) && ( !bSearchDone ))
+    while (( pExtension != NULL ) && (pResult == NULL))
     {
-
+        //VERBOSE(VB_UPNP, QString("UPNP Search : Searching for : %1  / ObjectID : %2").arg(pExtension->m_sExtensionId).arg(request.m_sObjectId));
         pResult = pExtension->Search( &request );
 
-        if (pResult != NULL)
+    	pExtension = m_extensions.next();
+    }
+
+    if (pResult != NULL)
+    {
+        eErrorCode  = pResult->m_eErrorCode;
+        sErrorDesc  = pResult->m_sErrorDesc;
+
+        if (eErrorCode == UPnPResult_Success)
         {
-            eErrorCode  = pResult->m_eErrorCode;
-            sErrorDesc  = pResult->m_sErrorDesc;
-
-            if (eErrorCode == UPnPResult_Success)
-            {
-                nNumberReturned = pResult->m_List.count();
-                nTotalMatches   = pResult->m_nTotalMatches;
-                nUpdateID       = pResult->m_nUpdateID;
-                sResultXML      = pResult->GetResultXML();
-                bSearchDone = true;
-            }
-
-            delete pResult;
+            nNumberReturned = pResult->m_List.count();
+            nTotalMatches   = pResult->m_nTotalMatches;
+            nUpdateID       = pResult->m_nUpdateID;
+            sResultXML      = pResult->GetResultXML();
+            //bSearchDone = true;
         }
 
-        pExtension = m_extensions.next();
-
+        delete pResult;
     }
 
     // nUpdateID       = 0;
@@ -559,13 +567,24 @@ void UPnpCDS::HandleGetSystemUpdateID( HTTPRequest *pRequest )
 //
 /////////////////////////////////////////////////////////////////////////////
 
+bool UPnpCDSExtension::IsBrowseRequestForUs( UPnpCDSRequest *pRequest )
+{
+    if (!pRequest->m_sObjectId.startsWith( m_sExtensionId, true ))
+        return false;
+
+    return true;
+}
+
+/////////////////////////////////////////////////////////////////////////////
+//
+/////////////////////////////////////////////////////////////////////////////
+
 UPnpCDSExtensionResults *UPnpCDSExtension::Browse( UPnpCDSRequest *pRequest )
 {
-
     // -=>TODO: Need to add Filter & Sorting Support.
     // -=>TODO: Need to add Sub-Folder/Category Support!!!!!
 
-    if (! pRequest->m_sObjectId.startsWith( m_sExtensionId, true ))
+    if (!IsBrowseRequestForUs( pRequest ))
         return( NULL );
 
     // ----------------------------------------------------------------------
@@ -587,8 +606,22 @@ UPnpCDSExtensionResults *UPnpCDSExtension::Browse( UPnpCDSRequest *pRequest )
 
     if (pResults != NULL)
     {
+
         if (key)  
             idPath.last().append(QString("=%1").arg(key)); 
+        else 
+        {
+            if (pRequest->m_sObjectId.contains("item"))
+            {
+                idPath = QStringList::split( " ", idPath[idPath.count() - 2] );
+                idPath = QStringList::split( "?", idPath[0] );
+
+                idPath = idPath[0];
+
+                if (idPath[0].startsWith("Id"))
+                    idPath[0] = QString("item=%1").arg(idPath[0].right(idPath[0].length() - 2));
+            }
+        }
 
         QString sLast = idPath.last();
 
@@ -615,15 +648,31 @@ UPnpCDSExtensionResults *UPnpCDSExtension::Browse( UPnpCDSRequest *pRequest )
 //
 /////////////////////////////////////////////////////////////////////////////
 
+bool UPnpCDSExtension::IsSearchRequestForUs( UPnpCDSRequest *pRequest )
+{
+    if ( !m_sClass.startsWith( pRequest->m_sSearchClass ))
+        return false;
+
+    return true;
+}
+
+/////////////////////////////////////////////////////////////////////////////
+//
+/////////////////////////////////////////////////////////////////////////////
+
 UPnpCDSExtensionResults *UPnpCDSExtension::Search( UPnpCDSRequest *pRequest )
 {
     // -=>TODO: Need to add Filter & Sorting Support.
     // -=>TODO: Need to add Sub-Folder/Category Support!!!!!
 
     QStringList sEmptyList;
+    VERBOSE(VB_UPNP, QString("UPnpCDSExtension::Search : m_sClass = %1 : m_sSearchClass = %2").arg(m_sClass).arg(pRequest->m_sSearchClass));
 
-     if ( !m_sClass.startsWith( pRequest->m_sSearchClass ))
+    if ( !IsSearchRequestForUs( pRequest ))
+    {
+        VERBOSE( VB_UPNP, QString("UPnpCDSExtension::Search - Not For Us : m_sClass = %1 : m_sSearchClass = %2").arg(m_sClass).arg(pRequest->m_sSearchClass));
         return NULL;
+    }
 
     UPnpCDSExtensionResults *pResults = new UPnpCDSExtensionResults();
 
@@ -688,6 +737,7 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessRoot( UPnpCDSRequest
 
         case CDS_BrowseDirectChildren:
         {
+		VERBOSE(VB_UPNP, "CDS_BrowseDirectChildren");
             pResults->m_nUpdateID     = 1;
             pResults->m_nTotalMatches = nRootCount ;
             
@@ -702,7 +752,6 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessRoot( UPnpCDSRequest
                 for (short nIdx = nStart; nIdx < nEnd; nIdx++)
                 {
                     UPnpCDSRootInfo *pInfo = GetRootInfo( nIdx );
-
                     if (pInfo != NULL)
                     {
 
@@ -713,7 +762,7 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessRoot( UPnpCDSRequest
                                                             QObject::tr( pInfo->title ), 
                                                             m_sExtensionId );
 
-                        pItem->SetChildCount( GetDistinctCount( pInfo->column ) );
+                        pItem->SetChildCount( GetDistinctCount( pInfo ) );
 
                         pResults->Add( pItem );
                     }
@@ -764,7 +813,7 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessAll ( UPnpCDSRequest
                                                     QObject::tr( pInfo->title ), 
                                                     m_sExtensionId );
 
-                pItem->SetChildCount( GetDistinctCount( pInfo->column ) );
+                pItem->SetChildCount( GetDistinctCount( pInfo ) );
 
                 pResults->Add( pItem ); 
             }
@@ -804,39 +853,68 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessItem( UPnpCDSRequest
     // ----------------------------------------------------------------------
     //
     // ----------------------------------------------------------------------
-    
-    if ( pRequest->m_eBrowseFlag == CDS_BrowseMetadata )
+   // VERBOSE(VB_UPNP, QString("UPnpCDSExtension::ProcessItem : %1").arg(idPath
+    switch( pRequest->m_eBrowseFlag )
     {
-        // --------------------------------------------------------------
-        // Return 1 Item
-        // --------------------------------------------------------------
-
-        QStringMap  mapParams;
-        QString     sParams = idPath.last().section( '?', 1, 1 );
-            
-        sParams.replace(QRegExp( "&amp;"), "&" ); 
-
-        HTTPRequest::GetParameters( sParams, mapParams );
-
-        MSqlQuery query(MSqlQuery::InitCon());
-
-        if (query.isConnected())                                                           
+	case CDS_BrowseMetadata:
         {
-            BuildItemQuery( query, mapParams );
+            // --------------------------------------------------------------
+            // Return 1 Item
+            // --------------------------------------------------------------
 
-            query.exec();
+            QStringMap  mapParams;
+            QString     sParams = idPath.last().section( '?', 1, 1 );
+            sParams.replace(QRegExp( "&amp;"), "&" ); 
 
-            if (query.isActive() && query.size() > 0)
+            HTTPRequest::GetParameters( sParams, mapParams );
+
+            MSqlQuery query(MSqlQuery::InitCon());
+
+            if (query.isConnected())                                                           
             {
-                if ( query.next() )
-                {
-                    pRequest->m_sObjectId = RemoveToken( "/", pRequest->m_sObjectId, 1 );
+                BuildItemQuery( query, mapParams );
 
-                    AddItem( pRequest->m_sObjectId, pResults, false, query );
-                    pResults->m_nTotalMatches = 1;
+                query.exec();
+    
+                if (query.isActive() && query.size() > 0)
+                {
+                    if ( query.next() )
+                    {
+                        pRequest->m_sObjectId = RemoveToken( "/", pRequest->m_sObjectId, 1 );
+
+                        AddItem( pRequest->m_sObjectId, pResults, false, query );
+                        pResults->m_nTotalMatches = 1;
+                    }
                 }
             }
         }
+        case CDS_BrowseDirectChildren:
+        {
+
+	    QStringList tokens;
+
+	    if (pRequest->m_sObjectId.length() > 0)
+            {
+                if (pRequest->m_sObjectId.contains("="))
+                    tokens = QStringList::split( "=", pRequest->m_sObjectId );
+	        else 
+	        {
+	             tokens =  QStringList::split( "/", pRequest->m_sObjectId );
+
+	             tokens = QStringList::split( " ", tokens[tokens.count() - 1] );
+                     tokens = QStringList::split( "?", tokens[0] );
+
+                     if (tokens[0].startsWith("Id"))
+                         tokens[0] = tokens[0].right(tokens[0].length() - 2);
+
+	        }	
+            }
+
+            pRequest->m_sParentId = tokens.last();
+            CreateItems( pRequest, pResults, 0, "", false );
+            break;
+        }
+
     }
 
     return pResults;
@@ -910,7 +988,7 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessKey( UPnpCDSRequest          *
                                                                 query.value(1).toString(), 
                                                                 pRequest->m_sParentId );
 
-                            pItem->SetChildCount( GetDistinctCount( pInfo->column  ));
+                            pItem->SetChildCount( GetDistinctCount( pInfo ));
 
                             pResults->Add( pItem ); 
                         }
@@ -921,6 +999,7 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessKey( UPnpCDSRequest          *
 
             case CDS_BrowseDirectChildren:
             {
+		
                 CreateItems( pRequest, pResults, nNodeIdx, sKey, true );
 
                 break;
@@ -968,7 +1047,7 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessContainer( UPnpCDSRequest
                                                 QObject::tr( pInfo->title ), 
                                                 m_sExtensionId );
 
-            pItem->SetChildCount( GetDistinctCount( pInfo->column ));
+            pItem->SetChildCount( GetDistinctCount( pInfo ));
 
             pResults->Add( pItem ); 
 
@@ -977,7 +1056,7 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessContainer( UPnpCDSRequest
 
         case CDS_BrowseDirectChildren:
         {
-            pResults->m_nTotalMatches = GetDistinctCount( pInfo->column );
+            pResults->m_nTotalMatches = GetDistinctCount( pInfo );
             pResults->m_nUpdateID     = 1;
 
             if (pRequest->m_nRequestedCount == 0) 
@@ -1040,10 +1119,13 @@ UPnpCDSExtensionResults *UPnpCDSExtension::ProcessContainer( UPnpCDSRequest
 //
 /////////////////////////////////////////////////////////////////////////////
 
-int UPnpCDSExtension::GetDistinctCount( const QString &sColumn )
+int UPnpCDSExtension::GetDistinctCount( UPnpCDSRootInfo *pInfo )
 {
     int nCount = 0;
 
+    if ((pInfo == NULL) || (pInfo->column == NULL))
+        return 0;
+
     MSqlQuery query(MSqlQuery::InitCon());
 
     if (query.isConnected())
@@ -1053,17 +1135,17 @@ int UPnpCDSExtension::GetDistinctCount( const QString &sColumn )
 
         QString sSQL;
         
-        if (sColumn == "*")
+        if (strncmp( pInfo->column, "*", 1) == 0)
         {
             sSQL = QString( "SELECT count( %1 ) FROM %2" )
-                      .arg( sColumn )
-                      .arg( GetTableName( sColumn ));
+                      .arg( pInfo->column )
+                      .arg( GetTableName( pInfo->column ));
         }
         else
         {
             sSQL = QString( "SELECT count( DISTINCT %1 ) FROM %2" )
-                      .arg( sColumn )
-                      .arg( GetTableName( sColumn ) );
+                      .arg( pInfo->column )
+                      .arg( GetTableName( pInfo->column ) );
         }
 
         query.prepare( sSQL );
@@ -1158,7 +1240,7 @@ void UPnpCDSExtension::CreateItems( UPnpCDSRequest          *pRequest,
         }
 
         QString sSQL = QString( "%1 %2 LIMIT %3, %4" )
-                          .arg( GetItemListSQL( pInfo->column ) )
+                          .arg( GetItemListSQL( pInfo->column )  )
                           .arg( sWhere )
                           .arg( pRequest->m_nStartingIndex  )
                           .arg( pRequest->m_nRequestedCount );
@@ -1169,8 +1251,9 @@ void UPnpCDSExtension::CreateItems( UPnpCDSRequest          *pRequest,
 
         if (query.isActive() && query.size() > 0)
         {
-            while(query.next())
+            while(query.next()) 
                 AddItem( pRequest->m_sObjectId, pResults, bAddRef, query );
+
         }
     }
 }
