@@ -13,6 +13,12 @@ using namespace std;
 #include "programinfo.h"
 #include "util.h"
 #include "previewgenerator.h"
+#include "storagegroup.h"
+#include "backendutil.h"
+#include "compat.h"
+
+#define LOC QString("EncoderLink(%1): ").arg(m_capturecardnum)
+#define LOC_ERR QString("EncoderLink(%1) Error: ").arg(m_capturecardnum)
 
 /**
  * \class EncoderLink
@@ -36,7 +42,7 @@ EncoderLink::EncoderLink(int capturecardnum, PlaybackSock *lsock,
                          QString lhostname)
     : m_capturecardnum(capturecardnum), sock(lsock), hostname(lhostname),
       freeDiskSpaceKB(-1), tv(NULL), local(false), locked(false), 
-      chanid(""), recordfileprefix(QString::null)
+      chanid("")
 {
     endRecordingTime = QDateTime::currentDateTime().addDays(-2);
     startRecordingTime = endRecordingTime;
@@ -51,11 +57,10 @@ EncoderLink::EncoderLink(int capturecardnum, PlaybackSock *lsock,
 EncoderLink::EncoderLink(int capturecardnum, TVRec *ltv)
     : m_capturecardnum(capturecardnum), sock(NULL), hostname(QString::null),
       freeDiskSpaceKB(-1), tv(ltv), local(true), locked(false), 
-      chanid(""), recordfileprefix(QString::null)
+      chanid("")
 {
     endRecordingTime = QDateTime::currentDateTime().addDays(-2);
     startRecordingTime = endRecordingTime;
-    recordfileprefix = gContext->GetSetting("RecordFilePrefix");
 }
 
 /** \fn EncoderLink::~EncoderLink()
@@ -83,23 +88,23 @@ void EncoderLink::SetSocket(PlaybackSock *lsock)
     sock = lsock;
 }
 
-/** \fn EncoderLink::IsBusy()
+/** \fn EncoderLink::IsBusy(TunedInputInfo*,int)
  *  \brief  Returns true if the recorder is busy, or will be within the 
- *          next 5 seconds.
- *  \sa IsBusyRecording(), TVRec::IsBusy()
+ *          next time_buffer seconds.
+ *  \sa IsBusyRecording(void), TVRec::IsBusy(TunedInputInfo*)
  */
-bool EncoderLink::IsBusy(void)
+bool EncoderLink::IsBusy(TunedInputInfo *busy_input, int time_buffer)
 {
     if (local)
-        return tv->IsBusy();
+        return tv->IsBusy(busy_input, time_buffer);
 
     if (sock)
-        return sock->IsBusy(m_capturecardnum);
+        return sock->IsBusy(m_capturecardnum, busy_input, time_buffer);
 
     return false;
 }
 
-/** \fn EncoderLink::IsBusyRecording()
+/** \fn EncoderLink::IsBusyRecording(void)
  *  \brief Returns true if the TVRec state is in a recording state.
  *
  *   Contrast with IsBusy() which returns true if a recording is pending
@@ -143,6 +148,27 @@ TVState EncoderLink::GetState(void)
     return retval;
 }
 
+/** \fn EncoderLink::GetFlags(void) const
+ *  \brief Returns the flag state of the recorder.
+ *  \sa TVRec::GetFlags(void) const, \ref recorder_subsystem
+ */
+uint EncoderLink::GetFlags(void) const
+{
+    uint retval = 0;
+
+    if (!IsConnected())
+        return retval;
+
+    if (local)
+        retval = tv->GetFlags();
+    else if (sock)
+        retval = sock->GetEncoderState(m_capturecardnum);
+    else
+        VERBOSE(VB_IMPORTANT, LOC_ERR + "GetFlags failed");
+
+    return retval;
+}
+
 /** \fn EncoderLink::IsRecording(const ProgramInfo *rec)
  *  \brief Returns true if rec is scheduled for recording.
  *  \param rec Recording to check.
@@ -199,18 +225,21 @@ bool EncoderLink::MatchesRecording(const ProgramInfo *rec)
     return retval;
 }
 
-/** \fn EncoderLink::RecordPending(const ProgramInfo*, int)
+/** \fn EncoderLink::RecordPending(const ProgramInfo*, int, bool)
  *  \brief Tells TVRec there is a pending recording "rec" in "secsleft" seconds.
+ *
  *  \param rec      Recording to make.
  *  \param secsleft Seconds to wait before starting recording.
+ *  \param hasLater If true, a later non-conflicting showing is available.
  *  \sa StartRecording(const ProgramInfo*), CancelNextRecording(bool)
  */
-void EncoderLink::RecordPending(const ProgramInfo *rec, int secsleft)
+void EncoderLink::RecordPending(const ProgramInfo *rec, int secsleft,
+                                bool hasLater)
 {
     if (local)
-        tv->RecordPending(rec, secsleft);
+        tv->RecordPending(rec, secsleft, hasLater);
     else if (sock)
-        sock->RecordPending(m_capturecardnum, rec, secsleft);
+        sock->RecordPending(m_capturecardnum, rec, secsleft, hasLater);
 }
 
 /** \fn EncoderLink::WouldConflict(const ProgramInfo*)
@@ -229,45 +258,28 @@ bool EncoderLink::WouldConflict(const ProgramInfo *rec)
     return false;
 }
 
-/** \fn EncoderLink::GetFreeDiskSpace(long long&,long long&,bool)
- *  \brief Returns total used and free disk space in Kilobytes
- *         and if use_cache is false updates the cached value.
- *
- *   May be a local or remote query.
- *
- *  \param use_cache if true, then a cached value is used.
- */
-long long EncoderLink::GetFreeDiskSpace(long long &totalKB, long long &usedKB,
-                                        bool use_cache)
+bool EncoderLink::CheckFile(ProgramInfo *pginfo)
 {
-    if (!use_cache)
+    if (sock)
+        return sock->CheckFile(pginfo);
+    else
     {
-        if (local)
-            freeDiskSpaceKB = getDiskSpace(recordfileprefix, totalKB, usedKB);
-        else if (sock)
-        {
-            sock->GetFreeDiskSpace(totalKB, usedKB);
-            freeDiskSpaceKB = totalKB - usedKB;
-            if ((-1 == totalKB) || (-1 == usedKB))
-                freeDiskSpaceKB = -1;
-        }
+        pginfo->pathname = GetPlaybackURL(pginfo);
+        if (pginfo->pathname.left(1) == "/")
+            return true;
     }
-    return freeDiskSpaceKB;
+    return false;
 }
 
-/** \fn EncoderLink::GetFreeDiskSpace(bool use_cache=false)
- *  \brief Returns total used and free disk space in Kilobytes
- *         and if use_cache is false updates the cached value.
+/**
+ *  \brief Appends total and used disk space in Kilobytes
  *
- *   May be a local or remote query.
- *
- *  \param use_cache if true, then a cached value is used.
+ *  \param o_strlist list to append to
  */
-long long EncoderLink::GetFreeDiskSpace(bool use_cache)
+void EncoderLink::GetDiskSpace(QStringList &o_strlist)
 {
-    long long totalKB = 0LL;
-    long long usedKB  = 0LL;
-    return GetFreeDiskSpace(totalKB, usedKB, use_cache);
+    if (sock)
+        sock->GetDiskSpace(o_strlist);
 }
 
 /** \fn EncoderLink::GetMaxBitrate()
@@ -323,79 +335,12 @@ int EncoderLink::LockTuner()
     return m_capturecardnum;
 }
 
-/** \fn  EncoderLink::HasEnoughFreeSpace(const ProgramInfo*, bool)
- *  \brief Returns true if there is enough free space for entire recording.
- *
- *   May be a local or remote query.
- *
- *   If AutoExpire is enabled, and the encoder is connected, this always
- *   returns true. If it is not enabled this function estimates the size
- *   of the recording, and only records if there is enough space left on
- *   the file system for both the recording and the "Extra Space" the 
- *   user desires on the file system.
- *
- *
- *   Note: If AutoExpire is not enabled, the calculation is only valid
- *         when one recording per file system is in progress at any one
- *         time.
- *
- *  \param rec Recording we wish to make, used to determine length of recording.
- *  \param try_to_use_cache If true we try the the free space calculation with
- *                          the cached disk free space, and only query the
- *                          backend for the true free space value, if the
- *                          cached value tells us we don't have enough free
- *                          space to fit the recording.
- */
-bool EncoderLink::HasEnoughFreeSpace(const ProgramInfo *rec, bool try_to_use_cache)
-{
-    if (!IsConnected())
-        return false;
-
-    // if auto expire is enabled, the space will be freed just in time...
-    if (gContext->GetNumSetting("AutoExpireMethod", 1))
-        return true;
-
-    // if we can't determine the free space, assume it's just an fstat problem.
-    long long freeSpaceKB = GetFreeDiskSpace(try_to_use_cache);
-    if (freeSpaceKB<0)
-        return true;
-    // shrink by "Extra Space" desired for other non-MythTV activities.
-    freeSpaceKB -= 1024 * 1024 *
-        gContext->GetNumSetting("AutoExpireExtraSpace", 0);
-    // if already below threshold we can't record anything.
-    if (freeSpaceKB<0)
-        return false;
-
-    // if we can't determine the bitrate assume it's ATSC 1080i.
-    long long maxBitrate = GetMaxBitrate();
-    if (maxBitrate<=0)
-        maxBitrate = 19500000LL;
-
-    // determine maximum program size
-    size_t bitRateKBperMin = (((size_t)maxBitrate)*((size_t)15))>>11;
-    size_t progLengthMin = (rec->startts.secsTo(rec->endts)+59)/60;
-    size_t programSizeKB = bitRateKBperMin * progLengthMin;
-    programSizeKB += programSizeKB>>2; // + 25%
-    VERBOSE(VB_IMPORTANT,
-            QString("Estimated program length: %1 minutes, "
-                    "size: %2 MB, free space: %3 MB")
-            .arg(progLengthMin).arg((long)(programSizeKB/1024))
-            .arg((long)(freeSpaceKB/1024)));
-    
-    // if program is smaller than available space, then ok
-    // if not ok, and we were using cache, try again with measured values
-    bool ok = ((long long)programSizeKB < freeSpaceKB);
-    if (!ok && try_to_use_cache)
-        ok = HasEnoughFreeSpace(rec, false);
-    return ok;
-}
-
 /** \fn EncoderLink::StartRecording(const ProgramInfo*)
  *  \brief Tells TVRec to Start recording the program "rec" as soon as possible.
  *
  *  \return +1 if the recording started successfully,
  *          -1 if TVRec is busy doing something else, 0 otherwise.
- *  \sa RecordPending(const ProgramInfo*, int), StopRecording()
+ *  \sa RecordPending(const ProgramInfo*, int, bool), StopRecording()
  */
 RecStatusType EncoderLink::StartRecording(const ProgramInfo *rec)
 {
@@ -571,35 +516,35 @@ void EncoderLink::FrontendReady(void)
 
 /** \fn EncoderLink::CancelNextRecording(bool)
  *  \brief Tells TVRec to cancel the next recording.
- *         <b>This only works on local recorders.</b>
  *
  *   This is used when the user is watching "Live TV" and does not
  *   want to allow the recorder to be taken for a pending recording.
  *
- *  \sa RecordPending(const ProgramInfo*,int)
+ *  \sa RecordPending(const ProgramInfo*, int, bool)
  */
 void EncoderLink::CancelNextRecording(bool cancel)
 {
     if (local)
         tv->CancelNextRecording(cancel);
     else
-        VERBOSE(VB_IMPORTANT, "Should be local only query: CancelNextRecording");
+        sock->CancelNextRecording(m_capturecardnum, cancel);
 }
 
-/** \fn EncoderLink::SpawnLiveTV(LiveTVChain*, bool)
+/** \fn EncoderLink::SpawnLiveTV(LiveTVChain*, bool, QString)
  *  \brief Tells TVRec to Spawn a "Live TV" recorder.
  *         <b>This only works on local recorders.</b>
  *
  *  \param chain The LiveTV chain to use
+ *  \param startchan The channel the LiveTV should start with
  *  \param pip Tells TVRec's RingBuffer that this is for a Picture in Picture di
 splay.
- *  \sa TVRec::SpawnLiveTV(LiveTVChain*,bool),
- *      RemoteEncoder::SpawnLiveTV(LiveTVChain*,bool)
+ *  \sa TVRec::SpawnLiveTV(LiveTVChain*,bool,QString),
+ *      RemoteEncoder::SpawnLiveTV(QString,bool,QString)
  */
-void EncoderLink::SpawnLiveTV(LiveTVChain *chain, bool pip)
+void EncoderLink::SpawnLiveTV(LiveTVChain *chain, bool pip, QString startchan)
 {
     if (local)
-        tv->SpawnLiveTV(chain, pip);
+        tv->SpawnLiveTV(chain, pip, startchan);
     else
         VERBOSE(VB_IMPORTANT, "Should be local only query: SpawnLiveTV");
 }
@@ -656,20 +601,31 @@ void EncoderLink::SetLiveRecording(int recording)
         VERBOSE(VB_IMPORTANT, "Should be local only query: SetLiveRecording");
 }
 
-/** \fn EncoderLink::GetConnectedInputs(void) const
- *  \brief Returns TVRec's recorders connected inputs.
- *         <b>This only works on local recorders.</b>
- *
- *  \sa TVRec::GetConnectedInputs(void) const
+/**
+ *  \brief Tells TVRec where to put the next LiveTV recording.
  */
-QStringList EncoderLink::GetConnectedInputs(void) const
+void EncoderLink::SetNextLiveTVDir(QString dir)
 {
-    QStringList list;
+    if (local)
+        tv->SetNextLiveTVDir(dir);
+    else
+        sock->SetNextLiveTVDir(m_capturecardnum, dir);
+}
+
+/** \fn EncoderLink::GetFreeInputs(const vector<uint>&) const
+ *  \brief Returns TVRec's recorders connected inputs.
+ *
+ *  \sa TVRec::GetFreeInputs(const vector<uint>&) const
+ */
+vector<InputInfo> EncoderLink::GetFreeInputs(
+    const vector<uint> &excluded_cardids) const
+{
+    vector<InputInfo> list;
 
     if (local)
-        list = tv->GetConnectedInputs();
+        list = tv->GetFreeInputs(excluded_cardids);
     else
-        VERBOSE(VB_IMPORTANT, "Should be local only query: GetConnectedInputs");
+        list = sock->GetFreeInputs(m_capturecardnum, excluded_cardids);
 
     return list;
 }
@@ -695,7 +651,7 @@ QString EncoderLink::GetInput(void) const
  *
  *   You must call PauseRecorder(void) before calling this.
  *
- *  \param input Input to switch to, or "SwitchToNectInput".
+ *  \param input Input to switch to, or "SwitchToNextInput".
  *  \return input we have switched to
  *  \sa TVRec::SetInput(QString)
  */
@@ -907,32 +863,4 @@ bool EncoderLink::SetChannelInfo(uint chanid, uint sourceid,
                               callsign, channum, channame, xmltv);
 }
 
-/** \fn EncoderLink::GetScreenGrab(const ProgramInfo*,const QString&,int,int&,int&,int&,float&)
- *  \brief Returns a PIX_FMT_RGBA32 buffer containg a frame from the video.
- *         <b>This only works on local recorders.</b>
- *  \param pginfo       Recording to grab from.
- *  \param filename     File containing recording.
- *  \param secondsin    Seconds into the video to seek before capturing a frame.
- *  \param bufferlen    Returns size of buffer returned (in bytes).
- *  \param video_width  Returns width of frame grabbed.
- *  \param video_height Returns height of frame grabbed.
- *  \param video_aspect Returns the aspect ratio of frame grabbed.
- *  \return Buffer allocated with new containing frame in RGBA32 format if
- *          successful, NULL otherwise.
- */
-char *EncoderLink::GetScreenGrab(const ProgramInfo *pginfo,
-                                 const QString &filename, 
-                                 int secondsin, int &bufferlen, 
-                                 int &video_width, int &video_height,
-                                 float &video_aspect)
-{
-    if (local)
-    {
-        return PreviewGenerator::GetScreenGrab(
-            pginfo, filename, secondsin, bufferlen, 
-            video_width, video_height, video_aspect);
-    }
-    VERBOSE(VB_IMPORTANT, "Should be local only query: GetScreenGrab");
-    return NULL;
-}
-
+/* vim: set expandtab tabstop=4 shiftwidth=4: */
